from Crypto.PublicKey import RSA
import os
from maica.maica_utils import get_outer_path

def generate_rsa_keys():

    prv_path = get_outer_path('keys/prv.key')
    pub_path = get_outer_path('keys/pub.key')

    try:
        with open(prv_path, "r") as privkey_file:
            privkey = privkey_file.read()
        with open(pub_path, "r") as pubkey_file:
            pubkey = pubkey_file.read()
        
        print("[maica-keys-init] Keys exist already, skipping...")
        return
    
    except:
        print("[maica-keys-init] Keys not exist, creating...")
        os.makedirs(get_outer_path('keys'))
    
        key = RSA.generate(2048)
        
        private_key = key.export_key()

        with open(prv_path, "wb") as priv_file:
            priv_file.write(private_key)
        
        public_key = key.publickey().export_key()

        with open(pub_path, "wb") as pub_file:
            pub_file.write(public_key)

        print("[maica-keys-init] Keys generated successfully, store with care!")

# 使用示例
if __name__ == "__main__":
    generate_rsa_keys()  # 使用默认路径



import json
import random
import traceback
import asyncio

from openai.types.chat import ChatCompletion, ChatCompletionMessage
from typing import *
from maica.mfocus.mfocus_sfe import SfBoundCoroutine
from maica.mtrigger.mtrigger_sfe import MtBoundCoroutine
from maica.maica_utils import *

class MTriggerCoroutine(SideFunctionCoroutine):
    def __init__(self, fsc: FullSocketsContainer, mt_inst: MtBoundCoroutine, sf_inst: Optional[SfBoundCoroutine]=None):
        super().__init__(fsc, sf_inst, mt_inst)

    def _construct_tools(self):
        self.tools = []
        trigger_list = self.mt_inst.get_valid_triggers()
        for trigger in trigger_list:
            match trigger.template:
                case 'common_affection_template':
                    current_aff = self.sf_inst.read_from_sf('mas_affection') if self.sf_inst else None
                    if current_aff:
                        current_aff = int(float(current_aff))
                        current_aff_str = f', 当前好感度是{current_aff}' if self.settings.basic.target_lang == 'zh' else f', current affection is {current_aff}'
                    else:
                        current_aff_str = ''
                    description_aff = f"当你认为对用户的好感度应发生变化时调用该工具{current_aff_str}. 例如, 当用户发送夸赞, 关心, 或表现明显的情感时可以调用该工具." if self.settings.basic.target_lang == 'zh' else f"Call this tool to change affection when you think you should{current_aff_str}. For example, you can call this tool when user is complimenting, showing care, or expressing notable emotion."
                    self.tools.append(
                        {
                            "name": trigger.name,
                            "description": description_aff,
                            "parameters": {
                                "type": "object",
                                "properties": {
                                    "affection": {
                                        "type": "number",
                                        "description": "输出正数以增加好感, 负数以减少好感. 单次最大好感增幅约为3, 一般增幅约为1. 例如, 称赞你的容貌可以增加0.8, 表达爱情的短句可以增加1.5, 表达爱情的长句则可以增加3.0.\n仅当用户故意冒犯时考虑输出负数." if self.settings.basic.target_lang == 'zh' else "Emit positive number to increase affection, negative to decrease affection. The maximum increase should be around 3, while it's normally around 1.\nFor example, a compliment about your beauty could result in plus 0.8, a short sentence expressing love could result in plus 1.5, and a long phrase expressing love could result in plus 3.0.\nYou shouldn't emit a negative value unless user is offending you intensively.",
                                        "example_value": "0.25"
                                    }
                                },
                                "required": [
                                    "affection"
                                ],
                                "optional": [
                                ]
                            }
                        }
                    )
                case 'common_switch_template':
                    item_common_name = trigger.exprop.item_name.zh if self.settings.basic.target_lang == 'zh' else trigger.exprop.item_name.en
                    curr_item = trigger.exprop.curr_item
                    item_list = trigger.exprop.item_list
                    if curr_item:
                        current_choice_str = f', 当前的{item_common_name}是{curr_item}' if self.settings.basic.target_lang == 'zh' else f', current {item_common_name} is {curr_item}'
                    else:
                        current_choice_str = ''
                    description_switch = f'调用该工具以切换{item_common_name}{current_choice_str}.' if self.settings.basic.target_lang == 'zh' else f'Call this tool to switch {item_common_name}{current_choice_str}.'
                    self.tools.append(
                        {
                            "name": trigger.name,
                            "description": description_switch,
                            "parameters": {
                                "type": "object",
                                "properties": {
                                    "selection": {
                                        "type": "string",
                                        "description": f'根据用户的要求, 从以下{item_common_name}中选出最合适的一项:\n{item_list}\n只选择, 不作改动. 如果没有任何一项符合要求, 则回答false.' if self.settings.basic.target_lang == 'zh' else f"According to user's request, choose a proper {item_common_name} from the following list:\n{item_list}\nMake your choice without modifying any item. If nothing in list matches user's request, return false.",
                                        "example_value": random.choice(item_list)
                                    }
                                },
                                "required": [
                                    "selection"
                                ],
                                "optional": [
                                ]
                            }
                        }
                    )
                    if trigger.exprop.suggestion:
                        self.tools[-1].update(
                            {
                                "parameters": {
                                    "properties": {
                                        "suggestion": {
                                            "type": "string",
                                            "description": f'若你在selection中回答了false, 你可以在此回答列表之外的{item_common_name}, 以满足用户要求.' if self.settings.basic.target_lang == 'zh' else f'If you chose false in the selection section, you can reply a {item_common_name} that\'s not in the list, which can satisfy the user\'s request.'
                                        }
                                    },
                                    "optional": [
                                        "suggestion"
                                    ]
                                }
                            }
                        )
                case 'common_meter_template':
                    meter_name = trigger.exprop.item_name.zh if self.settings.basic.target_lang == 'zh' else trigger.exprop.item_name.en
                    curr_value = trigger.exprop.curr_value
                    lower, upper = trigger.exprop.value_limits
                    if curr_value:
                        current_value_str = f', 当前的{meter_name}值是{curr_value}' if self.settings.basic.target_lang == 'zh' else f', current {meter_name} value is {curr_value}'
                    else:
                        current_value_str = ''
                    description_meter = f'调用该工具以调整{meter_name}{current_value_str}' if self.settings.basic.target_lang == 'zh' else f'Call this tool to adjust {meter_name}{current_value_str}'
                    self.tools.append(
                        {
                            "name": trigger.name,
                            "description": description_meter,
                            "parameters": {
                                "type": "object",
                                "properties": {
                                    "value": {
                                        "type": "number",
                                        "description": f'根据用户的要求, 在{lower}到{upper}之间为{meter_name}选择一个合适的值. 如果合适的值不存在, 则回答false.' if self.settings.basic.target_lang == 'zh' else f"According to user's request, choose a proper value for {meter_name} in range {lower} to {upper}. If no value in range matches the request, return false.",
                                        "example_value": "0.25"
                                    }
                                },
                                "required": [
                                    "value"
                                ],
                                "optional": [
                                ]
                            }
                        }
                    )
                case 'customized':
                    trigger_name = trigger.exprop.item_name.zh if self.settings.basic.target_lang == 'zh' else trigger.exprop.item_name.en
                    self.tools.append(
                        {
                            "name": trigger.name,
                            "description": f'调用该工具以触发{trigger_name}.' if self.settings.basic.target_lang == 'zh' else f'Call this tool to trigger {trigger_name}.',
                            "parameters": {
                                "type": "object",
                                "properties": {
                                },
                                "required": [
                                ],
                                "optional": [
                                ]
                            }
                        }
                    )

        self.tools.append(
            {
                "name": "agent_finished",
                "description": f"若你已调用了所有其它必要的工具, 则调用该工具以表示你已完成." if self.settings.basic.target_lang == 'zh' else f"Call this tool after you've finished calling every other necessary tool, so we know you're done.",
                "parameters": {
                    "type": "object",
                    "properties": {
                    },
                    "required": [
                    ],
                    "optional": [
                    ]
                }
            }
        )

        self.tools = alt_tools(self.tools)

    async def _construct_query(self, user_input=None, tool_input=None, tool_id=None):
        await super()._construct_query(user_input, tool_input, tool_id, 'post')

    async def triggering(self, input, output):
        try:

            # Prepare the first query first
            self._construct_tools()
            await self._construct_query()

            # This is a little bit special
            self.serial_messages.extend([{'role': 'user', 'content': input}, {'role': 'assistant', 'content': output}])
            user_instruct_input = '观察以上对话历史记录, 依据你上一次作出的回应调用工具. 除好感外, 不要调用未经用户明确指示的工具, 每个工具最多调用一次.' if self.settings.basic.target_lang == 'zh' else 'Observe the chat history and make tool calls according to your last reply. Do not use tools except affection, unless user requested directly. Each tool can only be used once at most.'
            await self._construct_query(user_input=user_instruct_input)

            cycle = 0; ending = False
            all_tool_count = 0
            while cycle <= 3 and not ending:

                # Sanity check
                cycle += 1

                resp_content, resp_tools = await self._send_query()
                await messenger(self.websocket, 'maica_mtrigger_toolchain', f'\nMTrigger toolchain {cycle} round responded, response is:\n{resp_content}\nAnalyzing response...', code='200')
                tool_seq = 0
                if resp_tools:
                    for resp_tool in resp_tools:

                        # Tool parallel support
                        tool_seq += 1; all_tool_count += 1
                        tool_id, tool_type, tool_func_name, tool_func_args = resp_tool.id, resp_tool.type, resp_tool.function.name, resp_tool.function.arguments
                        await messenger(info=f'\nCalling parallel tool {tool_seq}/{len(resp_tools)}:\n{resp_tool}\nSending trigger...', type=MsgType.PRIM_LOG)

                        if tool_func_name == 'agent_finished':
                            ending = True
                            break
                        else:
                            trigger_signal = {tool_func_name: try_load_json(tool_func_args)}
                            await messenger(self.websocket, 'maica_mtrigger_trigger', trigger_signal, '200', type='carriage')

                            machine = f'{tool_func_name}已被调用过并生效' if self.settings.basic.target_lang == 'zh' else f'{tool_func_name} has been called already and taking effect'
                            await self._construct_query(tool_input=machine, tool_id=tool_id)

                else:
                    ending = True
                    
                await messenger(None, 'maica_mtrigger_round_finish', f'MTrigger toolchain {cycle} round finished, ending is {str(ending)}.')
            
            await messenger(self.websocket, 'maica_mtrigger_done', f'MTrigger ended with {all_tool_count} triggers sent', '1001')

        except Exception as e:
            if isinstance(e, CommonMaicaException):
                raise e
            else:
                raise CommonMaicaError(str(e), '500', 'maica_mtrigger_critical')



import asyncio
import json

from typing import *
from Crypto.Random import random as CRANDOM

from maica.mfocus import MFocusCoroutine, SfBoundCoroutine
from maica.mtrigger import MTriggerCoroutine, MtBoundCoroutine
from maica.maica_utils import *

def pkg_init_maica_nows():
    global PROMPT_ZC, PROMPT_ZW, PROMPT_EC, PROMPT_EW
    PROMPT_ZC = load_env('PROMPT_ZC')
    PROMPT_ZW = load_env('PROMPT_ZW')
    PROMPT_EC = load_env('PROMPT_EC')
    PROMPT_EW = load_env('PROMPT_EW')
    maica_assert(str(PROMPT_ZC) and str(PROMPT_ZW) and str(PROMPT_EC) and str(PROMPT_EW), 'prompt')

class NoWsCoroutine(AsyncCreator):
    """
    Not actually no-ws, but ws can be None.
    Also no AI socket.
    """

    # To be populated or not
    sf_inst = None
    mt_inst = None
    mfocus_coro = None
    mtrigger_coro = None

    # Initialization

    def __init__(self, auth_pool: DbPoolCoroutine, maica_pool: DbPoolCoroutine, websocket=None, online_dict = {}):
        self.auth_pool, self.maica_pool, self.websocket, self.online_dict = auth_pool, maica_pool, websocket, online_dict
        self.traceray_id = str(CRANDOM.randint(0,9999999999)).zfill(10)
        self.settings = MaicaSettings()
        self.fsc = FullSocketsContainer(self.websocket, self.traceray_id, self.settings, self.auth_pool, self.maica_pool)

    async def _ainit(self):
        self.hasher = await AccountCursor.async_create(self.settings, self.auth_pool, self.maica_pool)

    def _check_essentials(self) -> None:
        if not self.settings.verification.user_id:
            raise MaicaPermissionError('Essentials not implemented', '403', 'common_essentials_missing')

    def flush_traceray(self) -> None:
        """Generates a new traceray_id for this instance."""
        self.traceray_id = str(CRANDOM.randint(0,9999999999)).zfill(10)
        self.fsc.rsc.traceray_id = self.traceray_id

    async def _create_session(self, user_id=None, chat_session_num=None, content=None) -> int:
        user_id = self.settings.verification.user_id if not user_id else user_id
        chat_session_num = self.settings.temp.chat_session if not chat_session_num else chat_session_num
        sql_expression = "INSERT INTO chat_session (user_id, chat_session_num, content) VALUES (%s, %s, %s)"
        return await self.maica_pool.query_modify(expression=sql_expression, values=(user_id, chat_session_num, content))

    async def _jsonify_chat_session(self, text) -> list:
        try:
            if text:
                return json.loads(f'[{text}]')
            else:
                return []
        except Exception as e:
            raise MaicaDbError(f'Chat session not JSON: {str(e)}', '500', 'maica_db_corruption')

    def _flattern_chat_session(self, content_json: list) -> str:
        if content_json:
            return json.dumps(content_json, ensure_ascii=False).strip('[').strip(']')
        else:
            return None

    async def _try_repair_chat_session(self, text) -> list:
        """Therotically shouldn't happen."""
        text = text.strip(', ').strip(' ,')
        content_json = await self._jsonify_chat_session(text)
        if content_json[0]['role'] != 'system':
            content_json.insert(0, {"role": "system", "content": await self.gen_system_prompt()})
        while content_json[1]['role'] != 'user':
            content_json.pop(1)
        return content_json

    async def _chop_session(self, chat_session_id, content) -> tuple[int, str]:
        max_length_ascii = self.settings.basic.max_length * 3
        warn_length_ascii = int(max_length_ascii * (2/3))
        len_content_actual = len(content.encode()) - len(json.loads(f'[{content}]')) * 31
        if len_content_actual >= max_length_ascii:

            # First we check if there is a cchop avaliable
            sql_expression_1 = 'SELECT archive_id, content FROM cchop_archived WHERE chat_session_id = %s AND archived = 0'
            result = await self.maica_pool.query_get(expression=sql_expression_1, values=(chat_session_id, ))
            if result:
                archive_id, archive_content = result
            else:
                archive_id = None; archive_content = ''

            cutting_mat = json.loads(f"[{content}]")
            while len_content_actual >= warn_length_ascii or cutting_mat[1]['role'] == "assistant":
                # Since pos 0 is reserved for system
                if archive_content:
                    archive_content = archive_content + ', '
                popped_dict = cutting_mat.pop(1)
                archive_content = archive_content + json.dumps(popped_dict, ensure_ascii=False)
                len_content_actual -= (len(json.dumps(popped_dict, ensure_ascii=False).encode()) - 31)
            content = self._flattern_chat_session(cutting_mat)

            if archive_id:
                sql_expression_2 = "UPDATE cchop_archived SET content = %s WHERE archive_id = %s" if len(archive_content) <= 100000 else "UPDATE cchop_archived SET content = %s, archived = 1 WHERE archive_id = %s"
                await self.maica_pool.query_modify(expression=sql_expression_2, values=(archive_content, archive_id))
            else:
                sql_expression_2 = "INSERT INTO cchop_archived (chat_session_id, content, archived) VALUES (%s, %s, 0)" if len(archive_content) <= 100000 else "INSERT INTO cchop_archived (chat_session_id, content, archived) VALUES (%s, %s, 1)"
                await self.maica_pool.query_modify(expression=sql_expression_2, values=(chat_session_id, archive_content))

            cut_status = 1
        elif len_content_actual >= warn_length_ascii:
            cut_status = 2
        else:
            cut_status = 0
        return cut_status, content
    
    async def rw_chat_session(self, irwa='r', content_append: list=None, system_prompt: str=None, chat_session_num=None) -> tuple[int, int | list]:
        """A common way to operate chat sessions."""
        self._check_essentials()

        if not chat_session_num:
            chat_session_num = self.settings.temp.chat_session
        else:
            maica_assert(1 <= chat_session_num <= 9, "chat_session")

        sql_expression_1 = "SELECT chat_session_id, content FROM chat_session WHERE user_id = %s AND chat_session_num = %s"
        result = await self.maica_pool.query_get(expression=sql_expression_1, values=(self.settings.verification.user_id, chat_session_num))
        if result:
            chat_session_id, content_original = result
        else:
            chat_session_id = None; content_original = ''

        content_append = self._flattern_chat_session(content_append)
        if not content_append:
            content_finale = content_original
        elif content_original:
            content_finale = content_original + ', ' + content_append
        else:
            content_finale = content_append

        if irwa == 'i':

            # By 'i' we mean initiate, so we ensure this session has a prompt
            # Here we do not write user input to protect data integrity
            content_original_json = await self._jsonify_chat_session(content_original)
            content_finale_json = await self._jsonify_chat_session(content_finale)
            if not system_prompt:
                system_prompt = await self.gen_system_prompt()

            for j in [content_original_json, content_finale_json]:
                if j:
                    if j[0]['role'] == 'system':
                        j[0]['content'] = system_prompt
                    else:
                        j.insert(0, {"role": "system", "content": system_prompt})
                else:
                    j.append({"role": "system", "content": system_prompt})

            content_insert = self._flattern_chat_session(content_original_json)
            if not chat_session_id:
                chat_session_id = await self._create_session(content=content_insert, chat_session_num=chat_session_num)
            else:
                sql_expression_2 = "UPDATE chat_session SET content = %s WHERE chat_session_id = %s"
                await self.maica_pool.query_modify(expression=sql_expression_2, values=(content_insert, chat_session_id))
            return chat_session_id, content_finale_json

        elif irwa == 'r':
            if not chat_session_id:
                chat_session_id = await self._create_session(chat_session_num=chat_session_num)
            return chat_session_id, await self._jsonify_chat_session(content_finale)
        
        elif irwa == 'w':
            if not chat_session_id:
                chat_session_id = await self._create_session(content=content_append, chat_session_num=chat_session_num)
            else:
                sql_expression_2 = "UPDATE chat_session SET content = %s WHERE chat_session_id = %s"
                await self.maica_pool.query_modify(expression=sql_expression_2, values=(content_append, chat_session_id))
            return chat_session_id, 0

        elif irwa == 'a':
            cut_status, content_finale = await self._chop_session(chat_session_id, content_finale)
            if not chat_session_id:
                chat_session_id = await self._create_session(content=content_finale, chat_session_num=chat_session_num)
            else:
                sql_expression_2 = "UPDATE chat_session SET content = %s WHERE chat_session_id = %s"
                await self.maica_pool.query_modify(expression=sql_expression_2, values=(content_finale, chat_session_id))
            return chat_session_id, cut_status

    async def reset_chat_session(self, chat_session_num=None, content_new=None) -> bool:
        """The difference between this and rw_chat_session is that this one archives."""
        self._check_essentials()

        if not chat_session_num:
            chat_session_num = self.settings.temp.chat_session
        else:
            maica_assert(1 <= chat_session_num <= 9, "chat_session")

        sql_expression_1 = "SELECT chat_session_id, content FROM chat_session WHERE user_id = %s AND chat_session_num = %s"
        result = await self.maica_pool.query_get(expression=sql_expression_1, values=(self.settings.verification.user_id, chat_session_num))
        if result:
            chat_session_id, content_archive = result
            sql_expression_2 = "UPDATE chat_session SET content = %s WHERE chat_session_id = %s"
            await self.maica_pool.query_modify(expression=sql_expression_2, values=(content_new, chat_session_id))
            sql_expression_3 = "INSERT INTO csession_archived (chat_session_id, content) VALUES (%s, %s)"
            await self.maica_pool.query_modify(expression=sql_expression_3, values=(chat_session_id, content_archive))
            return True
        else:
            chat_session_id = await self._create_session(chat_session_num=chat_session_num, content=content_new)
            return False
        
    async def restore_chat_session(self, content_restore, chat_session_num=None) -> bool:
        """Basically not necessary to exist but who knows..."""
        self._check_essentials()

        if not chat_session_num:
            chat_session_num = self.settings.temp.chat_session
        else:
            maica_assert(1 <= chat_session_num < 10, "chat_session")

        if not isinstance(content_restore, str):
            content_restore = self._flattern_chat_session(content_restore)

        return await self.reset_chat_session(chat_session_num=chat_session_num, content_new=content_restore)

    async def find_ms_cache(self, hash_identity) -> Optional[str]:
        """Find ms cache with corresponding prompt hash."""

        sql_expression_1 = "SELECT content FROM ms_cache WHERE hash = %s"
        result = await self.maica_pool.query_get(expression=sql_expression_1, values=(hash_identity, ))
        if result:
            await messenger(None, 'maica_spire_cache_hit', 'Hit a stored cache for MSpire', '200')
            return result[0]
        else:
            await messenger(None, 'maica_spire_cache_missed', 'No stored cache for MSpire', '200')
            return None

    async def store_ms_cache(self, hash_identity, content) -> int:
        """Store ms cache with prompt hash."""
        self._check_essentials()

        sql_expression_1 = "INSERT INTO ms_cache (user_id, hash, content) VALUES (%s, %s, %s)"
        spire_id = await self.maica_pool.query_modify(expression=sql_expression_1, values=(self.settings.verification.user_id, hash_identity, content))
        await messenger(None, 'maica_spire_cache_stored', 'Stored a cache for MSpire', '200')
        return spire_id

    async def gen_system_prompt(self, known_info=None, strict_conv=None) -> str:
        def _basic_gen_system(player_name, target_lang='zh', strict_conv=True):
            if target_lang == 'zh':
                if strict_conv:
                    system_init = PROMPT_ZC
                else:
                    system_init = PROMPT_ZW
            else:
                if strict_conv:
                    system_init = PROMPT_EC
                else:
                    system_init = PROMPT_EW
            system_init = system_init.format(player_name=player_name)
            return system_init

        self._check_essentials()
        player_name = '[player]'

        if self.settings.extra.sfe_aggressive:
            player_name_get = self.sf_inst.read_from_sf('mas_playername')
            if player_name_get:
                player_name = player_name_get
                if known_info:
                    known_info = ReUtils.re_sub_player_name.sub(player_name, known_info)

        if not strict_conv:
            strict_conv = self.settings.temp.strict_conv
        new_system = _basic_gen_system(player_name, self.settings.basic.target_lang, strict_conv)
        if self.settings.extra.nsfw_acceptive:
            new_system += f" 你应当允许用户提出私密性的要求, 并给出宽容接纳的正面回答." if self.settings.basic.target_lang == 'zh' else f" You should accept NSFW and private queries and respond positively with acceptance and tolerance."
        if known_info:
            new_system += f" 以下是一些相关信息, 你可以参考其中有价值的部分, 并用你自己的语言方式作答: {known_info}" if self.settings.basic.target_lang == 'zh' else f" Here are some information you can refer to, then make your answer in your own way: {known_info}"
        return new_system
    
    async def populate_auxiliary_inst(self):
        self.sf_inst, self.mt_inst = await asyncio.gather(SfBoundCoroutine.async_create(self.fsc), MtBoundCoroutine.async_create(self.fsc))
        self.mfocus_coro, self.mtrigger_coro = await asyncio.gather(MFocusCoroutine.async_create(self.fsc, self.sf_inst, self.mt_inst), MTriggerCoroutine.async_create(self.fsc, self.mt_inst, self.sf_inst))

    async def reset_auxiliary_inst(self):
        sb_list = []
        for sb_name in ['sf_inst', 'mt_inst', 'mfocus_coro', 'mtrigger_coro']:
            sb = getattr(self, sb_name)
            if sb:
                sb_list.append(sb.reset())
        await asyncio.gather(*sb_list)

    async def hash_and_login(self, access_token: str=None, logged_in_already: bool=False, check_online: bool=False) -> bool:
        """Use this to login a NoWs instance."""
        if not logged_in_already:
            verification_result = await self.hasher.hashing_verify(access_token)
        else:
            verification_result = [True, None]
        if verification_result[0]:

            # Account security check
            checked_status = await self.hasher.check_user_status(False, 'banned')
            if checked_status[0]:
                raise MaicaPermissionError('Account banned by MAICA', '403', 'maica_account_banned')

            # Cridential correct and not banned
            if check_online and not logged_in_already:
                if self.settings.verification.user_id in self.online_dict:
                    if load_env("KICK_STALE_CONNS") == "0":
                        self.settings.verification.reset()
                        raise MaicaConnectionWarning('A connection was established already and kicking not enabled', '406', 'maica_connection_reuse_denied')
                    else:
                        await messenger(self.websocket, "maica_connection_reuse_attempt", "A connection was established already", "300", self.traceray_id)
                        stale_fsc, stale_lock = self.online_dict[self.settings.verification.user_id]
                        try:
                            await messenger(stale_fsc.rsc.websocket, 'maica_connection_reuse_stale', 'A new connection has been established', '300', stale_fsc.rsc.traceray_id)
                            await stale_fsc.rsc.websocket.close(1000, 'Displaced as stale')
                        except Exception:
                            await messenger(None, 'maica_connection_stale_dead', 'The stale connection has died already', '204')
                        try:
                            self.online_dict.pop(self.settings.verification.user_id)
                        except Exception:
                            pass
                        async with stale_lock:
                            await messenger(None, 'maica_connection_stale_kicked', 'The stale connection is kicked', '204')
            return True

        else:
            if isinstance(verification_result[1], dict):
                if 'f2b' in verification_result[1]:
                    raise MaicaPermissionError(f'Account locked by Fail2Ban, {verification_result[1]['f2b']} seconds remaining', '429', 'maica_login_denied_fail2ban')
                elif 'necf' in verification_result[1]:
                    raise MaicaPermissionError(f'Account Email not verified, check inbox and retry', '401', 'maica_login_denied_email')
                elif 'pwdw' in verification_result[1]:
                    raise MaicaPermissionWarning(f'Password hashing failed {verification_result[1]['pwdw']} times, check password and retry', '403', 'maica_login_denied_password')
            elif isinstance(verification_result[1], CommonMaicaException):
                raise verification_result[1]
            else:
                raise MaicaPermissionError(verification_result[1], '400', 'maica_login_denied_rsa')
        return False


import asyncio
import datetime
import schedule

from typing import *
from maica.maica_utils import *

async def rotate_cache(maica_pool: DbPoolCoroutine):
    """Always provide a pool in production deployment!"""
    keep_time = load_env('ROTATE_MSCACHE')
    if int(keep_time):
        timestamp = datetime.datetime.now()
        sql_expression_1 = "SELECT spire_id, timestamp FROM ms_cache"
        sql_expression_2 = "DELETE FROM ms_cache WHERE spire_id = %s"
        result = await maica_pool.query_get(sql_expression_1)
        for row in result:
            if row[1] + datetime.timedelta(hours=int(keep_time)) <= timestamp:
                await maica_pool.query_modify(sql_expression_2, (row[0]))

def wrap_rotate_cache(maica_pool):
    asyncio.run(rotate_cache(maica_pool))

async def schedule_rotate_cache(**kwargs):
    maica_created = False
    if kwargs.get('maica_pool'):
        maica_pool = kwargs.get('maica_pool')
    else:
        maica_pool = await ConnUtils.maica_pool()
        maica_created = True

    await messenger(info="MAICA scheduler started!", type=MsgType.SYS)
    if load_env('ROTATE_MSCACHE') != '0':
        await messenger()
        schedule.every().day.at("04:00").do(wrap_rotate_cache, maica_pool=maica_pool)
    try:
        while True:
            schedule.run_pending()
            await asyncio.sleep(60)
    except BaseException as be:
        if isinstance(be, Exception):
            error = CommonMaicaError(str(be), '504')
            await messenger(error=error, no_raise=True)
    finally:
        if maica_created:
            await maica_pool.close()
            
        await messenger(info='\n', type=MsgType.PLAIN)
        await messenger(info="MAICA scheduler stopped!", type=MsgType.SYS)



from openai.types.chat import ChatCompletionMessage
from .connection_utils import *
from .container_utils import *
from maica.maica_utils import *
"""Import layer 5"""

class SideBoundCoroutine(AsyncCreator):
    """This is just a template. Do not initialize!"""
    DB_NAME = 'persistents'
    PRIM_KEY = 'persistent_id'
    FUNC_NAME = 'mfocus'
    EMPTY = {}

    @staticmethod
    def test_log_cache_stats(func):
        async def wrapper(*args, **kwargs):
            result = await func(*args, **kwargs)
            print(f'Cache expired: {result}')
            return result
        return wrapper

    def __init__(self, fsc: FullSocketsContainer) -> None:
        self.settings: MaicaSettings = fsc.maica_settings
        self.mfocus_conn: AiConnCoroutine = fsc.mfocus_conn
        self.websocket, self.traceray_id = fsc.rsc.websocket, fsc.rsc.traceray_id
        self.maica_pool = fsc.maica_pool
        self.sf_forming_buffer = self.EMPTY
        self.p_id = self.timestamp = None
        self.sf_content = self.EMPTY
        self.formed_info = None

    async def _ainit(self):
        await self.reset(force=True)

    def _cap_2(self, text) -> str:
        if len(text) >= 2:
            return text[:2].upper() + text[2:]
        else:
            return text.upper()
        
    def _add(self, data, ex) -> Union[list, dict]:
        data.update(ex) if isinstance(data, dict) else data.extend(ex)
            
    async def _find_session_or_default(self) -> int:
        sql_expression = f'SELECT {self.PRIM_KEY} FROM {self.DB_NAME} WHERE user_id = %s AND chat_session_num = %s'
        if not await self.maica_pool.query_get(sql_expression, (self.settings.verification.user_id, self.settings.temp.chat_session)):
            return 1
        else:
            return self.settings.temp.chat_session

    # @test_log_cache_stats
    async def _check_expired_or_not(self) -> bool:
        if self.p_id and self.timestamp and self.sf_content:
            sql_expression_1 = f'SELECT timestamp FROM {self.DB_NAME} WHERE {self.PRIM_KEY} = %s'
            try:
                result = await self.maica_pool.query_get(sql_expression_1, (self.p_id, ))
                new_timestamp = result[0]
                if new_timestamp == self.timestamp:
                    return False
            except Exception:
                if self.sf_content is self.EMPTY:
                    return False
                else:
                    return True
        return True

    async def reset(self, force=False) -> None:
        if  force or await self._check_expired_or_not():
            try:
                sql_expression = f'SELECT {self.PRIM_KEY}, content, timestamp FROM {self.DB_NAME} WHERE user_id = %s AND chat_session_num = %s'
                result = await self.maica_pool.query_get(sql_expression, (self.settings.verification.user_id, await self._find_session_or_default()))

                p_id, content, timestamp = result
                self.p_id, self.timestamp = p_id, timestamp
                self.sf_content = json.loads(content)
            except Exception:
                self.p_id = self.timestamp = None
                self.sf_content = self.EMPTY
                await messenger(self.websocket, f'{self.FUNC_NAME}_no_persistent', f'No persistent found for {self._cap_2(self.FUNC_NAME)}, using empty', '204', traceray_id=self.traceray_id)
            self.sf_forming_buffer = self.EMPTY
            self._add(self.sf_forming_buffer, self.sf_content)
        else:
            # If the cache is not expired, we just reuse it
            self.sf_forming_buffer = self.EMPTY
            self._add(self.sf_forming_buffer, self.sf_content)

class SideFunctionCoroutine(AsyncCreator):
    """This is just a template. Do not initialize!"""
    def __init__(self, fsc: FullSocketsContainer, sf_inst=None, mt_inst=None):
        self.settings = fsc.maica_settings
        self.websocket, self.traceray_id = fsc.rsc.websocket, fsc.rsc.traceray_id
        self.mcore_conn, self.mfocus_conn = fsc.mcore_conn, fsc.mfocus_conn
        self.sf_inst, self.mt_inst = sf_inst, mt_inst
        self.maica_pool = fsc.maica_pool

    async def _ainit(self):
        await self.reset()

    async def reset(self):
        """Caution: we should reset sf_inst and mt_inst here, but these are done more manually to prevent duplication."""
        self.tnd_aggressive = self.settings.extra.tnd_aggressive
        if self.settings.temp.ic_prep:
            self.tnd_aggressive = 2
        self.tools = []
        self.serial_messages = []

    async def full_reset(self):
        """This resets sf_inst and mt_inst too."""
        reset_list = []
        if self.sf_inst:
            reset_list.append(self.sf_inst.reset())
        if self.mt_inst:
            reset_list.append(self.mt_inst.reset())
        await asyncio.gather(*reset_list)
        await self.reset()

    async def _construct_query(self, user_input=None, tool_input=None, tool_id=None, pre_post: Literal['pre', 'post']=None):
        match pre_post:
            case 'pre':
                additive_setting = self.settings.extra.pre_additive
            case 'post':
                additive_setting = self.settings.extra.post_additive
            case _:
                raise MaicaInputError('Need type assignment on use', '500')
        if not self.serial_messages and additive_setting and 1 <= self.settings.temp.chat_session <= 9:
            sql_expression = 'SELECT content FROM chat_session WHERE user_id = %s AND chat_session_num = %s'
            result = await self.maica_pool.query_get(expression=sql_expression, values=(self.settings.verification.user_id, self.settings.temp.chat_session))
            if result and result[0]:
                res_list = json.loads(f'[{result[0]}]')
                lines_num = min(self.settings.extra.pre_additive * 2, len(res_list) - 1)
                message_additive = res_list[-lines_num:] if lines_num > 0 else []
                if message_additive:
                    self.serial_messages.extend(message_additive)
                    assert self.serial_messages[-1]['role'] == 'assistant', 'Additive got corrupted chat history'

        if user_input:

            # Having user input here suggests the last tool calls are over.
            # So we have to cleanup the thinking part and toolcalls.
            self.serial_messages = [msg for msg in self.serial_messages if msg.get('role') != 'tool']
            assistant_last_msg = ''
            for msg in self.serial_messages[::-1]:
                if isinstance(msg, ChatCompletionMessage):
                    assistant_last_msg = msg.content + assistant_last_msg
                    self.serial_messages.pop()
                elif msg.get('role') == 'assistant':
                    assistant_last_msg = msg.get('content') + assistant_last_msg
                    self.serial_messages.pop()
                else:
                    break

            # Then we peal off the thinking part
            assistant_last_msg = proceed_agent_response(assistant_last_msg)

            if assistant_last_msg:
                self.serial_messages.append({'role': 'assistant', 'content': assistant_last_msg})

            self.serial_messages.append({'role': 'user', 'content': user_input})

        elif tool_input:
            self.serial_messages.append({'role': 'tool', 'tool_call_id': tool_id, 'content': tool_input})

    async def _send_query(self) -> tuple[str, list]:
        completion_args = {
            "messages": self.serial_messages,
            "tools": self.tools,
            "temperature": 0.2,
            "seed": 42
        }

        resp = await self.mfocus_conn.make_completion(**completion_args)
        content, tool_calls = resp.choices[0].message.content, resp.choices[0].message.tool_calls

        if load_env('ALT_TOOLCALL') != '0':
            self.serial_messages.append(resp.choices[0].message)
        else:
            self.serial_messages.append({"role": "assistant", "content": content})

        return content, tool_calls



import re
import asyncio
import traceback
import functools
from maica.maica_utils import *
from googlesearch import asearch
from openai import AsyncOpenAI # type: ignore

async def internet_search(fsc: FullSocketsContainer, query, original_query):
    target_lang = fsc.maica_settings.basic.target_lang

    for tries in range(0, 3):
        try:

            # Here goes the search module
            # Highly unstable I would say
            # Fuck google
            results_async = asearch(query, advanced=True, proxy=load_env("PROXY_ADDR"))
            results_sync = []
            async for result_async in results_async:
                results_sync.append({"title": result_async.title, "text": result_async.description})
        except Exception:
            if tries < 2:
                await messenger(info=f'Search engine temporary failure, retrying {str(tries + 1)} time(s)')
                await asyncio.sleep(0.5)
            else:
                raise MaicaInternetWarning(f'Cannot get search result after {str(tries + 1)} times', '408')

    results_full = []
    results_short = []
    results_humane = ''
    rank = 0
    for item in results_sync:
        rank += 1
        title, text = clean_text(item['title']), clean_text(item['text'])
        results_full.append({'rank': rank, 'title': title, 'text': text})
        if rank <= 5:
            results_short.append({'rank': rank, 'title': title, 'text': text})
        if rank <= 3:
            results_humane += f'信息{rank}\n标题:{title}\n内容:{text}\n'

    await messenger(info=f'MFocus got {rank} information lines from search engine', type=MsgType.DEBUG)

    results_full_str = str(results_full).strip('[').strip(']')
    results_short_str = str(results_short).strip('[').strip(']')
    if not fsc.maica_settings.extra.esc_aggressive:
        return results_short_str, results_humane

    system_init = """你是一个人工智能助手, 你接下来会收到一个问题和一些来自互联网的信息.
以单行不换行的自然语言的形式, 回答用户的原始问题, 并整理提供相关的信息. 如果你最终认为提供的信息不足以作答, 返回false.
Begin!""" if target_lang == 'zh' else """You are a helpful assistant, now you will recieve a question and some information from the Internet.
Answer the question in a single line of natural sentence, and conclude and offer related information briefly. If you think the provided information is not enough finally, return false.
Begin!"""

    messages = [{'role': 'system', 'content': system_init}]
    messages.append({'role': 'user', 'content': f'question: {original_query}; information: {results_full_str}'})
    completion_args = {
        "messages": messages,
        "temperature": 0.2,
        "top_p": 0.6,
        "presence_penalty": -0.5,
        "frequency_penalty": 0.5,
        "seed": 42
    }

    resp = await fsc.mfocus_conn.make_completion(**completion_args)
    response = resp.choices[0].message.content
            
    await messenger(None, 'mfocus_internet_search', f"\nMFocus toolchain searching internet, response is:\n{response}\nEnd of MFocus toolchain searching internet", '201')
    
    answer_post_think = proceed_agent_response(response)
    return answer_post_think, answer_post_think

if __name__ == '__main__':
    pass


import aiomysql
import aiosqlite
import asyncio
import traceback
import json
from typing import *
from openai import AsyncOpenAI
from openai.types.chat import ChatCompletion
from maica.maica_utils import *
from .setting_utils import *
from .locater import *
"""Import layer 3"""

def pkg_init_connection_utils():
    global DB_ADDR, DB_USER, DB_PASSWORD, AUTH_DB, MAICA_DB, MCORE_ADDR, MCORE_KEY, MCORE_CHOICE, MFOCUS_ADDR, MFOCUS_KEY, MFOCUS_CHOICE, ConnUtils
    DB_ADDR = load_env('DB_ADDR')
    DB_USER = load_env('DB_USER')
    DB_PASSWORD = load_env('DB_PASSWORD')
    AUTH_DB = load_env('AUTH_DB')
    MAICA_DB = load_env('MAICA_DB')
    MCORE_ADDR = load_env('MCORE_ADDR')
    MCORE_KEY = load_env('MCORE_KEY')
    MCORE_CHOICE = load_env('MCORE_CHOICE')
    MFOCUS_ADDR = load_env('MFOCUS_ADDR')
    MFOCUS_KEY = load_env('MFOCUS_KEY')
    MFOCUS_CHOICE = load_env('MFOCUS_CHOICE')

    if DB_ADDR != "sqlite":
        """We suppose we're using MySQL."""
        async def auth_pool(ro=True):
            return await DbPoolCoroutine.async_create(
                host=DB_ADDR,
                db=AUTH_DB,
                user=DB_USER,
                password=DB_PASSWORD,
                ro=ro,
            )

        async def maica_pool(ro=False):
            return await DbPoolCoroutine.async_create(
                host=DB_ADDR,
                db=MAICA_DB,
                user=DB_USER,
                password=DB_PASSWORD,
                ro=ro,
            )
        
        async def basic_pool(ro=False):
            return await DbPoolCoroutine.async_create(
                host=DB_ADDR,
                db=None,
                user=DB_USER,
                password=DB_PASSWORD,
                ro=ro,
            )
    else:
        """We suppose we're using SQLite."""
        async def auth_pool(ro=True):
            return await SqliteDbPoolCoroutine.async_create(
                db=get_inner_path(AUTH_DB)
            )
        
        async def maica_pool(ro=False):
            return await SqliteDbPoolCoroutine.async_create(
                db=get_inner_path(MAICA_DB)
            )
        
        async def basic_pool(ro=False):
            """
            There's no host concept in SQLite, so no basic pool.
            """
            return None
    setattr(ConnUtils, 'auth_pool', auth_pool)
    setattr(ConnUtils, 'maica_pool', maica_pool)
    setattr(ConnUtils, 'basic_pool', basic_pool)

RETRY_TIMES = 3

def test_logger(func):
    async def wrapper(self, expression, values, *args, **kwargs):
        print(f'Query: {expression}\nValues: {values}')
        result = await func(self, expression, values, *args, **kwargs)
        print(f'Result: {result}')
        return result
    return wrapper

class DbPoolCoroutine(AsyncCreator):
    """Maintain a database connection pool so you don't have to."""
    def __init__(self, db, host, user, password, ro=False):
        self.db, self.host, self.user, self.password, self.ro = db, host, user, password, ro

    async def _ainit(self):
        self.pool: aiomysql.Pool = await aiomysql.create_pool(host=self.host, user=self.user, password=self.password, db=self.db)

    async def keep_alive(self):
        try:
            async with self.pool.acquire():
                pass
            return
        except Exception:
            await messenger(info=f"Recreating {self.db} pool since cannot acquire", type=MsgType.WARN)
            try:
                self.pool.close()
                await self._ainit()
            except Exception:
                raise MaicaDbError(f'Failure when trying reconnecting to {self.db}', '502', 'db_connection_failed')

    # @test_logger
    async def query_get(self, expression, values=None, fetchall=False) -> list:
        results = None
        for tries in range(RETRY_TIMES):
            try:
                await self.keep_alive()
                async with self.pool.acquire() as conn:
                    async with conn.cursor() as cur:
                        if not values:
                            await cur.execute(expression)
                        else:
                            await cur.execute(expression, values)
                        results = await cur.fetchone() if not fetchall else await cur.fetchall()
                break
            except Exception as e:
                if tries < RETRY_TIMES - 1:
                    await messenger(info=f'DB temporary failure, retrying {str(tries + 1)} time(s)')
                    await asyncio.sleep(0.5)
                else:
                    raise MaicaDbError(f'DB get query failure after {str(tries + 1)} times: {str(e)}', '502', 'db_get_failed')
        return results

    # @test_logger
    async def query_modify(self, expression, values=None, fetchall=False) -> int:
        if self.ro:
            raise MaicaDbError(f'DB marked as ro, no modify permitted', '511', 'db_modification_denied')
        lrid = None
        for tries in range(RETRY_TIMES):
            try:
                await self.keep_alive()
                async with self.pool.acquire() as conn:
                    async with conn.cursor() as cur:
                        if not values:
                            await cur.execute(expression)
                        else:
                            await cur.execute(expression, values)
                        await conn.commit()
                        lrid = cur.lastrowid
                break
            except Exception as e:
                if tries < RETRY_TIMES - 1:
                    await messenger(info=f'DB temporary failure, retrying {str(tries + 1)} time(s)')
                    await asyncio.sleep(0.5)
                else:
                    raise MaicaDbError(f'DB modify query failure after {str(tries + 1)} times: {str(e)}', '502', 'db_modify_failed')
        return lrid
    
    async def close(self):
        """These connection pools shouldn't be closed manually in runtime. We implement it anyway."""
        self.pool.close()
        await self.pool.wait_closed()

class SqliteDbPoolCoroutine(DbPoolCoroutine):
    """SQLite-specific database pool coroutine."""

    def __init__(self, db, host=None, user=None, password=None, ro=False):
        self.db_path = db
        self.ro = ro
        self.pool = None

    async def _ainit(self):
        """Initialize SQLite connection pool."""
        try:
            self.pool = aiosqlite.connect(self.db_path)
            await self.pool.__aenter__()
            if self.ro:
                await self.pool.execute("PRAGMA query_only = ON")
        except Exception as e:
            raise MaicaDbError(f'Failed to initialize SQLite connection: {str(e)}', '502', 'sqlite_init_failed')

    async def keep_alive(self):
        """Check and maintain SQLite connection."""
        try:
            await self.pool.execute("SELECT 1")
        except Exception:
            await messenger(info=f"Recreating {self.db} pool since cannot acquire", type=MsgType.WARN)
            try:
                if self.pool:
                    await self.pool.close()
                await self._ainit()
            except Exception:
                raise MaicaDbError(f'Failure when trying reconnecting to {self.db}', '502', 'db_connection_failed')

    @Decos.escape_sqlite_expression
    # @test_logger
    async def query_get(self, expression, values=None, fetchall=False) -> list:
        """Execute SELECT query on SQLite database."""
        results = None
        for tries in range(RETRY_TIMES):
            try:
                await self.keep_alive()
                if not values:
                    cursor = await self.pool.execute(expression)
                else:
                    cursor = await self.pool.execute(expression, values)
                results = await cursor.fetchone() if not fetchall else await cursor.fetchall()
                await cursor.close()
                break
            except Exception as e:
                if tries < RETRY_TIMES - 1:
                    await messenger(info=f'SQLite temporary failure, retrying {str(tries + 1)} time(s)')
                    await asyncio.sleep(0.5)
                else:
                    raise MaicaDbError(f'SQLite get query failure after {str(tries + 1)} times: {str(e)}', '502', 'sqlite_get_failed')
        return results

    @Decos.escape_sqlite_expression
    # @test_logger
    async def query_modify(self, expression, values=None, fetchall=False) -> int:
        """Execute INSERT/UPDATE/DELETE query on SQLite database."""
        if self.ro:
            raise MaicaDbError(f'DB marked as ro, no modify permitted', '511', 'sqlite_modification_denied')
        lrid = None
        for tries in range(RETRY_TIMES):
            try:
                await self.keep_alive()
                if not values:
                    cursor = await self.pool.execute(expression)
                else:
                    cursor = await self.pool.execute(expression, values)
                await self.pool.commit()
                lrid = cursor.lastrowid
                await cursor.close()
                break
            except Exception as e:
                if tries < RETRY_TIMES - 1:
                    await messenger(info=f'SQLite temporary failure, retrying {str(tries + 1)} time(s)')
                    await asyncio.sleep(0.5)
                else:
                    raise MaicaDbError(f'SQLite modify query failure after {str(tries + 1)} times: {str(e)}', '502', 'sqlite_modify_failed')
        return lrid

    async def close(self):
        """Close SQLite connection."""
        if self.pool:
            await self.pool.close()

class AiConnCoroutine(AsyncCreator):
    """Maintain an AI connection so you don't have to."""
    def __init__(self, api_key, base_url, name='mcore_cli', model: Union[int, str]=0):
        self.test = False
        self.api_key, self.base_url, self.name, self.model = api_key, base_url, name, model
        self.websocket = None
        self.traceray_id = ''

    async def _ainit(self):
        if not self.base_url:
            self.test = True
            return
        else:
            try:
                self._open_socket()
                await self.keep_alive()
            except Exception:
                self.test = True
                return

    def _open_socket(self):
        self.socket = AsyncOpenAI(api_key=self.api_key, base_url=self.base_url)

    def init_rsc(self, rsc: FscPlain.RealtimeSocketsContainer):
        """AiConn can actually work without rsc, so we make it individual."""
        self.websocket, self.traceray_id = rsc.websocket, rsc.traceray_id

    async def keep_alive(self):
        try:
            model_list = await self.socket.models.list()
            models = model_list.data
            if isinstance(self.model, int):
                self.model_actual = models[0].id
            else:
                self.model_actual = self.model
        except Exception:
            await messenger(None, f'{self.name}_reconn', f"Recreating {self.name} client since cannot conn", '301', type=MsgType.WARN)
            try:
                try:
                    await self.socket.close()
                except Exception:
                    pass
                self._open_socket()
            except Exception:
                raise MaicaResponseError(f'Failure when trying reconnecting to {self.name}', '502', f'{self.name}_connection_failed')
            
    async def make_completion(self, **kwargs) -> ChatCompletion:
        kwargs.update(
            {
                "model": self.model_actual
            }
        )
        for tries in range(RETRY_TIMES):
            try:
                await self.keep_alive()
                task_stream_resp = asyncio.create_task(self.socket.chat.completions.create(**kwargs))
                await task_stream_resp
                return task_stream_resp.result()
            except Exception as e:
                if tries < RETRY_TIMES - 1:
                    await messenger(info=f'Model temporary failure, retrying {str(tries + 1)} time(s)')
                    await asyncio.sleep(0.5)
                else:
                    raise MaicaResponseError(f'{self.name} AI query failure after {str(tries + 1)} times: {str(e)}', '502', f'{self.name}_query_failed')

class ConnUtils():
    """Just a wrapping for functions."""
    async def auth_pool(ro=True):
        """Dummy."""
    async def maica_pool(ro=False):
        """Dummy."""
    async def basic_pool(ro=False):
        """Dummy."""
    async def mcore_conn():
        return await AiConnCoroutine.async_create(
            api_key=MCORE_KEY,
            base_url=MCORE_ADDR,
            name='mcore_cli',
            model=MCORE_CHOICE if MCORE_CHOICE else 0,
        )

    async def mfocus_conn():
        return await AiConnCoroutine.async_create(
            api_key=MFOCUS_KEY,
            base_url=MFOCUS_ADDR,
            name='mfocus_cli',
            model=MFOCUS_CHOICE if MFOCUS_CHOICE else 0,
        )

async def validate_input(input: Union[str, dict, list], limit: int=4096, rsc: Optional[FscPlain.RealtimeSocketsContainer]=None, must: list=[], warn: list=[]) -> Union[dict, list]:
    """
    Mostly for ws.
    """
    if not input:
        raise MaicaInputWarning('Input is empty', '410', 'maica_input_validation_denied')
    
    if isinstance(input, str):
        if len(input) > limit:
            raise MaicaInputWarning('Input length exceeded', '413', 'maica_input_validation_denied')
        try:
            input_json = json.loads(input)
        except Exception:
            raise MaicaInputWarning('Request body not JSON', '400', 'maica_input_validation_denied')
    elif isinstance(input, dict | list):
        if len(str(input)) > limit:
            raise MaicaInputWarning('Input length exceeded', '413', 'maica_input_validation_denied')
        input_json = input
    else:
        raise MaicaInputError('Input must be string or JSON-like', '400', 'maica_input_validation_denied')

    if must:
        for mustkey in must:
            if input_json.get(mustkey) is None:
                raise MaicaInputWarning(f'Request contains no necessary {mustkey}', '405', 'maica_input_validation_denied')
    if warn:
        for warnkey in warn:
            if input_json.get(warnkey) is None:
                if rsc:
                    await messenger(rsc.websocket, 'maica_future_warning', f'Requests containing no {warnkey} will likely be deprecated in the future', '302', type=MsgType.WARN)
    
    return input_json




from .mpostal import make_postmail
from .mspire import make_inspire
from .wiki_scraping import get_page
from .weather_scraping import weather_api_get
from .enet_scraping import internet_search
from .ev_scraping import RegEvent, EventsCollection
from .post_proc import post_proc
from .nv_watcher import NvWatcher

__all__ = [
    'make_postmail',
    'make_inspire',
    'weather_api_get',
    'internet_search',
    'RegEvent',
    'EventsCollection',
    'post_proc',
    'NvWatcher',
    ]


import sys
import os
import inspect
"""Import layer 0"""

def locater():
    """
    Gets the project/container path.
    """
    is_frozen = getattr(sys, 'frozen', False)
    
    if is_frozen:
        absolute_path = os.path.abspath(sys.executable)
    else:
        absolute_path = os.path.abspath(inspect.getfile(locater))

    dirname = os.path.dirname(absolute_path)
    if not is_frozen:
        for i in range(2):
            dirname = os.path.dirname(dirname)
    
    return is_frozen, dirname

def get_inner_path(filename):
    frozen, base_path = locater()
    if frozen:
        filepath = os.path.join(base_path, filename)
    else:
        filepath = os.path.join(base_path, 'maica', filename)
    return filepath

def get_outer_path(filename):
    frozen, base_path = locater()
    return os.path.join(base_path, filename)

if __name__ == "__main__":
    is_frozen, self_path = locater()
    print(f"是否已被打包: {is_frozen}")
    print(f"函数所在文件绝对路径: {self_path}")


import asyncio
from typing import *
from maica.maica_utils import *
"""Import layer 2"""

PRV_LIST = ['_lock', 'lock']

class MaicaSettings():
    """All the per-client settings for MAICA."""

    class _common_funcs():
        """Just a template. Do not initialize!"""

        def __init__(self):
            self._lock = False

        def reset(self):
            self.__init__()

        def __call__(self):
            return {k.lstrip('_'): getattr(self, k.lstrip('_')) for k, _ in vars(self).items() if not k in PRV_LIST}

        def update(self, **kwargs):
            kwargs = {k: v for k, v in kwargs.items() if not k in PRV_LIST}
            accepted_params = 0
            for k, v in kwargs.items():
                if hasattr(self, f'_{k}'):
                    setattr(self, k, v)
                    accepted_params += 1
            return accepted_params

        @classmethod
        def default(cls, key=None):
            inst = cls()
            defaults = {k.lstrip('_'): v for k, v in vars(inst).items() if not k in PRV_LIST}
            if not key:
                return defaults
            else:
                return defaults[key]

    class _identity(_common_funcs):
        """Note that this identity is not verified and not safe to use in most cases. Use verification for those."""

        def __init__(self):
            super().__init__()
            self._user_id: int = None
            self._username: str = None
            self._nickname: Optional[str] = None
            self._email: str = None

        def _try_lock(self):
            pass

        def force_lock(self):
            self._lock = True

        @property
        @Decos.report_reading_error
        def user_id(self):
            assert self._user_id
            return self._user_id
        @user_id.setter
        @Decos.report_limit_error
        def user_id(self, v: int):
            v = int(v)
            assert v > 0
            self._user_id = v

        @property
        @Decos.report_reading_error
        def username(self):
            assert self._username
            return self._username
        @username.setter
        @Decos.report_limit_error
        def username(self, v: str):
            v = str(v)
            assert v
            self._username = v

        @property
        @Decos.report_reading_error
        def nickname(self):
            return self._nickname
        @nickname.setter
        @Decos.report_limit_error
        def nickname(self, v: Optional[str]):
            self._nickname = v

        @property
        @Decos.report_reading_error
        def email(self):
            assert self._email
            return self._email
        @email.setter
        @Decos.report_limit_error
        def email(self, v: str):
            v = str(v)
            assert v
            self._email = v

    class _verification(_identity):
        """Verified identity, safe to use."""
        def _try_lock(self):
            self._lock = True

    class _basic(_common_funcs):
        """Major params that decide MAICA's behavior."""

        def __init__(self):
            super().__init__()
            self._stream_output: bool = True
            self._deformation: bool = False
            self._enable_mf: bool = True
            self._enable_mt: bool = True
            self._sf_extraction: bool = True
            self._mt_extraction: bool = True
            self._target_lang: str = 'zh'
            self._max_length: int = 8192

        @property
        def stream_output(self):
            """Use stream output."""
            return self._stream_output
        @stream_output.setter
        @Decos.report_limit_warning
        def stream_output(self, v: Optional[bool]):
            if v is None:
                self._stream_output = self.default('stream_output')
            else:
                self._stream_output = bool(v)

        @property
        def deformation(self):
            """Deprecated."""
            return self._deformation
        @deformation.setter
        @Decos.report_limit_warning
        def deformation(self, v: Optional[bool]):
            if v is None:
                self._deformation = self.default('deformation')
            else:
                self._deformation = bool(v)

        @property
        def enable_mf(self):
            """Enable MFocus."""
            return self._enable_mf
        @enable_mf.setter
        @Decos.report_limit_warning
        def enable_mf(self, v: Optional[bool]):
            if v is None:
                self._enable_mf = self.default('enable_mf')
            else:
                self._enable_mf = bool(v)

        @property
        def enable_mt(self):
            """Enable MTrigger."""
            return self._enable_mt
        @enable_mt.setter
        @Decos.report_limit_warning
        def enable_mt(self, v: Optional[bool]):
            if v is None:
                self._enable_mt = self.default('enable_mt')
            else:
                self._enable_mt = bool(v)

        @property
        def sf_extraction(self):
            """Enable savefile extraction."""
            return self._sf_extraction
        @sf_extraction.setter
        @Decos.report_limit_warning
        def sf_extraction(self, v: Optional[bool]):
            if v is None:
                self._sf_extraction = self.default('sf_extraction')
            else:
                self._sf_extraction = bool(v)

        @property
        def mt_extraction(self):
            """Enable trigger extraction."""
            return self._mt_extraction
        @mt_extraction.setter
        @Decos.report_limit_warning
        def mt_extraction(self, v: Optional[bool]):
            if v is None:
                self._mt_extraction = self.default('mt_extraction')
            else:
                self._mt_extraction = bool(v)

        @property
        def target_lang(self):
            """Target language."""
            return self._target_lang
        @target_lang.setter
        @Decos.report_limit_warning
        def target_lang(self, v: Optional[str]):
            if v is None:
                self._target_lang = self.default('target_lang')
            else:
                self._target_lang = 'zh' if v == 'zh' else 'en'

        @property
        def max_length(self):
            """Max session length."""
            return self._max_length
        @max_length.setter
        @Decos.report_limit_warning
        def max_length(self, v: Optional[int]):
            if v is None:
                self._max_length = self.default('max_length')
            else:
                assert 512 <= int(v) <= 28672
                self._max_length = int(v)

    class _extra(_common_funcs):
        """Params that aren't that important, but affect MAICA's behavior."""

        def __init__(self):
            super().__init__()
            self._sfe_aggressive: bool = False
            self._mf_aggressive: bool = False
            self._tnd_aggressive: int = 1
            self._esc_aggressive: bool = True
            self._amt_aggressive: bool = True
            self._nsfw_acceptive: bool = True
            self._pre_additive: int = 0
            self._post_additive: int = 1
            self._tz: Optional[str] = None

        @property
        def sfe_aggressive(self):
            """Use name from savefile instead of [player] in prompts."""
            return self._sfe_aggressive
        @sfe_aggressive.setter
        @Decos.report_limit_warning
        def sfe_aggressive(self, v: Optional[bool]):
            if v is None:
                self._sfe_aggressive = self.default('sfe_aggressive')
            else:
                self._sfe_aggressive = bool(v)

        @property
        def mf_aggressive(self):
            """Use agent model's final output instead of instructed guidance."""
            return self._mf_aggressive
        @mf_aggressive.setter
        @Decos.report_limit_warning
        def mf_aggressive(self, v: Optional[bool]):
            if v is None:
                self._mf_aggressive = self.default('mf_aggressive')
            else:
                self._mf_aggressive = bool(v)

        @property
        def tnd_aggressive(self):
            """Add information to MFocus instructed guidance even if no tool used."""
            return self._tnd_aggressive
        @tnd_aggressive.setter
        @Decos.report_limit_warning
        def tnd_aggressive(self, v: Optional[int]):
            if v is None:
                self._tnd_aggressive = self.default('tnd_aggressive')
            else:
                assert 0 <= int(v) <= 3
                self._tnd_aggressive = int(v)

        @property
        def esc_aggressive(self):
            """Force agent to resort information acquired from Internet."""
            return self._esc_aggressive
        @esc_aggressive.setter
        @Decos.report_limit_warning
        def esc_aggressive(self, v: Optional[bool]):
            if v is None:
                self._esc_aggressive = self.default('esc_aggressive')
            else:
                self._esc_aggressive = bool(v)

        @property
        def amt_aggressive(self):
            """Add MTrigger toollist to MFocus tools for a precheck."""
            return self._amt_aggressive
        @amt_aggressive.setter
        @Decos.report_limit_warning
        def amt_aggressive(self, v: Optional[bool]):
            if v is None:
                self._amt_aggressive = self.default('amt_aggressive')
            else:
                self._amt_aggressive = bool(v)

        @property
        def nsfw_acceptive(self):
            """Alter prompt to ask model to handle toxic topics positively."""
            return self._nsfw_acceptive
        @nsfw_acceptive.setter
        @Decos.report_limit_warning
        def nsfw_acceptive(self, v: Optional[bool]):
            if v is None:
                self._nsfw_acceptive = self.default('nsfw_acceptive')
            else:
                self._nsfw_acceptive = bool(v)

        @property
        def pre_additive(self):
            """Add history rounds for MFocus to understand the conversation."""
            return self._pre_additive
        @pre_additive.setter
        @Decos.report_limit_warning
        def pre_additive(self, v: Optional[int]):
            if v is None:
                self._pre_additive = self.default('pre_additive')
            else:
                assert 0 <= int(v) <= 5
                self._pre_additive = int(v)

        @property
        def post_additive(self):
            """Add history rounds for MFocus to understand the conversation."""
            return self._post_additive
        @post_additive.setter
        @Decos.report_limit_warning
        def post_additive(self, v: Optional[int]):
            if v is None:
                self._post_additive = self.default('post_additive')
            else:
                assert 0 <= int(v) <= 5
                self._post_additive = int(v)

        @property
        def tz(self):
            """Timezone. This is not fully checked, double check before use."""
            return self._tz
        @tz.setter
        @Decos.report_limit_warning
        def tz(self, v: Optional[str]):
            if v is None:
                self._tz = self.default('tz')
            else:
                self._tz = str(v)

    class _super(_common_funcs):
        """Passthrough params to core LLM."""

        def __init__(self):
            super().__init__()
            self._max_tokens: int = 1600
            self._seed: int = None
            self._top_p: float = 0.7
            self._temperature: float = 0.22
            self._frequency_penalty: float = 0.44
            self._presence_penalty: float = 0.34

        @property
        def max_tokens(self):
            return self._max_tokens
        @max_tokens.setter
        @Decos.report_limit_warning
        def max_tokens(self, v: Optional[int]):
            if v is None:
                self._max_tokens = self.default('max_tokens')
            else:
                assert 0 < int(v) <= 2048
                self._max_tokens = int(v)

        @property
        def seed(self):
            return self._seed
        @seed.setter
        @Decos.report_limit_warning
        def seed(self, v: Optional[int]):
            if v is None:
                self._seed = self.default('seed')
            else:
                self._seed = int(v)

        @property
        def top_p(self):
            return self._top_p
        @top_p.setter
        @Decos.report_limit_warning
        def top_p(self, v: Optional[float]):
            if v is None:
                self._top_p = self.default('top_p')
            else:
                assert 0.1 <= float(v) <= 1.0
                self._top_p = float(v)

        @property
        def temperature(self):
            return self._temperature
        @temperature.setter
        @Decos.report_limit_warning
        def temperature(self, v: Optional[float]):
            if v is None:
                self._temperature = self.default('temperature')
            else:
                assert 0.0 <= float(v) <= 1.0
                self._temperature = float(v)

        @property
        def frequency_penalty(self):
            return self._frequency_penalty
        @frequency_penalty.setter
        @Decos.report_limit_warning
        def frequency_penalty(self, v: Optional[float]):
            if v is None:
                self._frequency_penalty = self.default('frequency_penalty')
            else:
                assert 0.0 <= float(v) <= 1.0
                self._frequency_penalty = float(v)

        @property
        def presence_penalty(self):
            return self._presence_penalty
        @presence_penalty.setter
        @Decos.report_limit_warning
        def presence_penalty(self, v: Optional[float]):
            if v is None:
                self._presence_penalty = self.default('presence_penalty')
            else:
                assert 0.0 <= float(v) <= 1.0
                self._presence_penalty = float(v)

    class _temp(_common_funcs):
        """Should be reset after each round of completion."""

        def __init__(self):
            super().__init__()
            self._chat_session = 0
            self._sf_extraction_once = False
            self._mt_extraction_once = False
            self._bypass_mf = False
            self._bypass_mt = False
            self._bypass_stream = False
            self._bypass_sup = False
            self._bypass_gen = False
            self._ic_prep = False
            self._strict_conv = True
            self._ms_cache = False

        @property
        def chat_session(self):
            return self._chat_session
        @chat_session.setter
        @Decos.report_limit_warning
        def chat_session(self, v: Optional[int]):
            if v is None:
                self._chat_session = self.default('chat_session')
            else:
                assert -1 <= int(v) <= 9
                self._chat_session = int(v)

        @property
        def sf_extraction_once(self):
            """Enable sf_extraction once while sf provided with query."""
            return self._sf_extraction_once
        @sf_extraction_once.setter
        @Decos.report_limit_warning
        def sf_extraction_once(self, v: Optional[bool]):
            if v is None:
                self._sf_extraction_once = self.default('sf_extraction_once')
            else:
                self._sf_extraction_once = bool(v)

        @property
        def mt_extraction_once(self):
            """Enable mt_extraction once while mt provided with query"""
            return self._mt_extraction_once
        @mt_extraction_once.setter
        @Decos.report_limit_warning
        def mt_extraction_once(self, v: Optional[bool]):
            if v is None:
                self._mt_extraction_once = self.default('mt_extraction_once')
            else:
                self._mt_extraction_once = bool(v)

        @property
        def bypass_mf(self):
            """Bypass MFocus once."""
            return self._bypass_mf
        @bypass_mf.setter
        @Decos.report_limit_warning
        def bypass_mf(self, v: Optional[bool]):
            if v is None:
                self._bypass_mf = self.default('bypass_mf')
            else:
                self._bypass_mf = bool(v)

        @property
        def bypass_mt(self):
            """Bypass MTrigger once."""
            return self._bypass_mt
        @bypass_mt.setter
        @Decos.report_limit_warning
        def bypass_mt(self, v: Optional[bool]):
            if v is None:
                self._bypass_mt = self.default('bypass_mt')
            else:
                self._bypass_mt = bool(v)

        @property
        def bypass_stream(self):
            """Bypass stream output once."""
            return self._bypass_stream
        @bypass_stream.setter
        @Decos.report_limit_warning
        def bypass_stream(self, v: Optional[bool]):
            if v is None:
                self._bypass_stream = self.default('bypass_stream')
            else:
                self._bypass_stream = bool(v)

        @property
        def bypass_sup(self):
            """Bypass super params once."""
            return self._bypass_sup
        @bypass_sup.setter
        @Decos.report_limit_warning
        def bypass_sup(self, v: Optional[bool]):
            if v is None:
                self._bypass_sup = self.default('bypass_sup')
            else:
                self._bypass_sup = bool(v)

        @property
        def bypass_gen(self):
            """Bypass generation once."""
            return self._bypass_gen
        @bypass_gen.setter
        @Decos.report_limit_warning
        def bypass_gen(self, v: Optional[bool]):
            if v is None:
                self._bypass_gen = self.default('bypass_gen')
            else:
                self._bypass_gen = bool(v)

        @property
        def ic_prep(self):
            """Adjust generation params once, basically for MPostal."""
            return self._ic_prep
        @ic_prep.setter
        @Decos.report_limit_warning
        def ic_prep(self, v: Optional[bool]):
            if v is None:
                self._ic_prep = self.default('ic_prep')
            else:
                self._ic_prep = bool(v)

        @property
        def strict_conv(self):
            """Strict conversation prompt."""
            return self._strict_conv
        @strict_conv.setter
        @Decos.report_limit_warning
        def strict_conv(self, v: Optional[bool]):
            if v is None:
                self._strict_conv = self.default('strict_conv')
            else:
                self._strict_conv = bool(v)

        @property
        def ms_cache(self):
            """Cache the MSpire response."""
            return self._ms_cache
        @ms_cache.setter
        @Decos.report_limit_warning
        def ms_cache(self, v: Optional[bool]):
            if v is None:
                self._ms_cache = self.default('ms_cache')
            else:
                self._ms_cache = bool(v)

    def __init__(self):
        self.identity, self.verification, self.basic, self.extra, self.super, self.temp = self._identity(), self._verification(), self._basic(), self._extra(), self._super(), self._temp()

    def __call__(self):
        d = self.identity()
        for k, v in self.verification().items():
            if v is not None:
                d[k] = v
        d.update(self.basic())
        d.update(self.extra())
        d.update(self.super())
        d.update(self.temp())
        return d

    def reset(self):
        self.__init__()

    def update(self, secure=None, **kwargs):
        """Used for handling manual settings."""
        if secure is False:
            accepted_params = self.identity.update(**kwargs)
        elif secure is True:
            accepted_params = self.verification.update(**kwargs)
        else:
            accepted_params = self.basic.update(**kwargs) + self.extra.update(**kwargs) + self.super.update(**kwargs)
            # We do not accept temps to be manually set
        return accepted_params





# Announcement:

# The code here is relatively simple, and have a lot of limitations
# I'm not devoting too much into this, since it's just a minor function of MAICA,
# and dealing with all these festivals and events is a massive work

# If you have corrections or improvements, I'd appreciate it a lot if you can PR
# them to me. FRs on improving this will not be considered, unless they're critical

import lunar_python
import datetime
import math
from typing import *
from maica.maica_utils import *

# lm = {
#     1: "正",
#     2: "二",
#     3: "三",
#     4: "四",
#     5: "五",
#     6: "六",
#     7: "七",
#     8: "八",
#     9: "九",
#     10: "十",
#     11: "十一",
#     12: "十二",
# }

# ld = {
#     1: "初一",
#     2: "初二",
#     3: "初三",
#     4: "初四",
#     5: "初五",
#     6: "初六",
#     7: "初七",
#     8: "初八",
#     9: "初九",
#     10: "初十",
#     11: "十一",
#     12: "十二",
#     13: "十三",
#     14: "十四",
#     15: "十五",
#     16: "十六",
#     17: "十七",
#     18: "十八",
#     19: "十九",
#     20: "二十",
#     21: "廿一",
#     22: "廿二",
#     23: "廿三",
#     24: "廿四",
#     25: "廿五",
#     26: "廿六",
#     27: "廿七",
#     28: "廿八",
#     29: "廿九",
#     30: "三十",
# }

# rlm = {v: k for k, v in lm.items()}
# rld = {v: k for k, v in ld.items()}

class RegEvent():

    type = None
    identity = None

    def __init__(self, md: Optional[str]=None, lmd: Optional[str]=None, mwd: Optional[str]=None, name: Optional[str]=None, ename: Optional[str]=None, importance=0, lasts=0):
        """
        md = 10_1 (month & day)
        lmd = 12_29 (lunar month & day)
        mwd = 4_2_3 (month & week & day) ! 0 for Sunday
        Do not present to target_lang == 'en' if no ename
        importance:
        2 => always inform
        1 => inform on target day (Monika likely cares)
        0 => no inform on tnd_aggressive
        lasts indicates how long the legal vacation is, 0 for none.
        The lasts var is only accurate for Chinese mainland.
        """
        if md:
            self.type = 'md'
            self.identity = md
        elif lmd:
            self.type = 'lmd'
            self.identity = lmd
        elif mwd:
            self.type = 'mwd'
            self.identity = mwd
        self.name, self.ename, self.importance, self.lasts = name, ename, importance, lasts

    def __str__(self):
        return self.name or self.ename
    
    def get_ymd(self, y):
        """
        Input ly if lmd, else y.
        Returns in solar.
        """
        match self.type:
            case "md":
                m, d = self.identity.split('_')
                solar = lunar_python.Solar.fromYmd(y, int(m), int(d))
            case "lmd":
                m, d = self.identity.split('_')
                solar = lunar_python.Lunar.fromYmd(y, int(m), int(d)).getSolar()
            case "mwd":
                m, w, wd = self.identity.split('_')
                solar_week: lunar_python.SolarWeek = lunar_python.SolarMonth.fromYm(y, int(m)).getWeeks(0)[int(w) - 1]
                solar = solar_week.getDays()[int(wd)]
        return solar.getYear(), solar.getMonth(), solar.getDay()

class EventsCollection():
    md_dict = {}
    lmd_dict = {}
    mwd_dict = {}

    @classmethod
    def _add_basic(cls, basic_events_list: list[RegEvent]):
        for event in basic_events_list:
            corr_dict = getattr(cls, f'{event.type}_dict')
            if not corr_dict.get(event.identity):
                corr_dict[event.identity] = [event]
            else:
                corr_dict[event.identity].append(event)

    def _add_vacations(self, y, m, d):
        """Note that this is ONLY valid for the target date!"""

        # Caution: This function may produce wrong results under certain circumstances:
        # - Target date in early January
        # - A long lunar vacation starts from late December and ends next year
        # - The vacation days registered early this solar year are actually from next year, 
        # which differs from the correct dates specifically for lunar events

        # Why we ignore this case:
        # - It's technically impossible to happen, since Spring festival is the only lunar
        # event with a long enough vacation, which never start before Solar 1.21
        # - It will be complex to design a prevention

        # If you want to customize this function, be aware of the mentioned possibility

        all_events_list = events_list + self.temp_events_list
        for event in all_events_list:
            if event.lasts and event.lasts > 1:
                if event.type == 'lmd':
                    y = lunar_python.Solar.fromYmd(y, m, d).getLunar().getYear()
                sy, sm, sd = event.get_ymd(y)
                event_date = datetime.datetime(sy, sm, sd)
                for i in range(1, event.lasts):
                    new_date = event_date + datetime.timedelta(days=i)
                    new_date_str = f"{new_date.month}_{new_date.day}"

                    # Up to now, there's actually no compatibility of vacations out of Chinese mainland, so we just disable their enames
                    new_event = RegEvent(md=new_date_str, name=f"{event.name}假期", ename=f"{event.ename} vacation" if False else None, importance=1, lasts=1)
                    self._add(new_event, vac=True)

    def __init__(self):
        self._reset()

    def _reset(self):
        self.temp_events_list = []
        self.md_dict_temp = {}
        self.lmd_dict_temp = {}
        self.mwd_dict_temp = {}
        self.vac_dict_temp = {}

    def _add(self, event: RegEvent, vac=False):
        corr_dict = getattr(self, f'{event.type}_dict_temp') if not vac else self.vac_dict_temp
        if not corr_dict.get(event.identity):
            corr_dict[event.identity] = [event]
        else:
            corr_dict[event.identity].append(event)

    def _find_qing_ming(self, y) -> int:
        """
        This is only accurate in 20th and 21st century.
        Fuck qm.
        """
        c = 4.81 if y >= 2001 else 5.59
        ymod = y % 100
        day = math.floor(ymod * 0.2422 + c) - math.floor(ymod / 4)
        day = max(4, min(6, day))
        return day
    
    def _get_last_weekday(self, y, m, wd) -> int:
        """Gets the last target weekday of a month."""
        month_last_day: lunar_python.Solar = lunar_python.SolarMonth.fromYm(y, m).getDays()[-1]
        month_last_week: lunar_python.SolarWeek = lunar_python.SolarWeek.fromYmd(month_last_day.getYear(), month_last_day.getMonth(), month_last_day.getDay(), 0)
        if len(month_last_week.getDaysInMonth()) < wd + 1:
            # So we move a week forwards
            month_last_2_week: lunar_python.SolarWeek = month_last_week.next(-1, separate_month=False)
            day: lunar_python.Solar = month_last_2_week.getDays()[wd]
        else:
            # Use last week
            day: lunar_python.Solar = month_last_week.getDays()[wd]
        return day.getDay()
    
    def _get_last_ld(self, y, m) -> int:
        """Gets the last day of a lunar month."""
        month: lunar_python.LunarMonth = lunar_python.LunarMonth.fromYm(y, m)
        return month.getDayCount()

    def find(self, y, m, d):
        y, m, d = int(y), int(m), int(d)

        common_date = datetime.datetime(y, m, d)
        ld = lunar_python.Lunar.fromDate(common_date)
        sw = lunar_python.SolarWeek.fromDate(common_date, 0)
        sd = lunar_python.Solar.fromDate(common_date)

        self.temp_events_list = [
            RegEvent(md=f"4_{self._find_qing_ming(y)}", name="清明节", importance=1, lasts=1),
            RegEvent(md=f"11_{self._get_last_weekday(y, 11, 4)}", name="感恩节", ename="Thanksgiving Day", importance=2),
            RegEvent(lmd=f"12_{self._get_last_ld(ld.getYear(), ld.getMonth())}", name="除夕", importance=2, lasts=1),
        ]

        self._add_vacations(y, m, d)

        for event in self.temp_events_list:
            self._add(event)

        md_identity = f"{m}_{d}"
        lmd_identity = f"{ld.getMonth()}_{ld.getDay()}"
        mwd_identity = f"{m}_{sw.getIndex()}_{sd.getWeek()}"

        conclusion = []
        conclusion.extend(self.md_dict.get(md_identity, []))
        conclusion.extend(self.lmd_dict.get(lmd_identity, []))
        conclusion.extend(self.mwd_dict.get(mwd_identity, []))
        conclusion.extend(self.md_dict_temp.get(md_identity, []))
        conclusion.extend(self.lmd_dict_temp.get(lmd_identity, []))
        conclusion.extend(self.mwd_dict_temp.get(mwd_identity, []))
        conclusion.extend(self.vac_dict_temp.get(md_identity, []))
        self._reset()

        return conclusion

# We only want to init them once, so we writing in root
events_list = [
    # By this method we assume the festivals are all year-even, which does not fit for qm, thanksgiving, and cx
    # So we're actually adding them in the find section
    RegEvent(md="1_1", name="元旦", ename="New Year's Day", importance=2, lasts=1),
    RegEvent(md="2_2", name="世界湿地日", ename="World Wetlands Day"),
    RegEvent(md="2_14", name="情人节", ename="Valentine's Day", importance=2),
    RegEvent(md="3_5", name="青年志愿者服务日"),
    RegEvent(md="3_8", name="妇女节", ename="Women's Day", lasts=1),
    RegEvent(md="3_12", name="植树节", ename="Arbor Day", importance=1),
    RegEvent(md="3_14", name="白色情人节", ename="White Day", importance=1),
    RegEvent(md="3_14", name="国际警察日", ename="International Policemen' Day"),
    RegEvent(md="3_15", name="世界消费者权益日", ename="World Consumer Right Day"),
    RegEvent(md="3_21", name="世界森林日", ename="World Forest Day", importance=1),
    RegEvent(md="3_21", name="世界睡眠日", ename="World Sleep Day"),
    RegEvent(md="3_22", name="世界水日", ename="World Water Day", importance=1),
    RegEvent(md="3_23", name="世界气象日", ename="World Meteorological Day", importance=1),
    RegEvent(md="3_24", name="世界防治结核病日", ename="World Tuberculosis Day"),
    RegEvent(md="4_1", name="愚人节", ename="April Fools' Day", importance=2),
    # RegEvent(md="4_5", name="清明节", importance=1, lasts=1),
    RegEvent(md="4_7", name="世界卫生日", ename="World Health Day", importance=1),
    RegEvent(md="4_22", name="世界地球日", ename="World Earth Day", importance=1),
    RegEvent(md="4_26", name="世界知识产权日", ename="World Intellectual Property Day"),
    RegEvent(md="5_1", name="劳动节", ename="Labour Day", importance=1, lasts=1),
    RegEvent(md="5_3", name="世界哮喘日", ename="World Asthma Day"),
    RegEvent(md="5_4", name="青年节", importance=1),
    RegEvent(md="5_8", name="世界红十字日", ename="World Red Cross Day", importance=1),
    RegEvent(md="5_12", name="国际护士节", ename="International Nurse Day"),
    RegEvent(md="5_15", name="国际家庭日", ename="International Family Day"),
    RegEvent(md="5_17", name="世界电信日", ename="World Telecommunications Day"),
    RegEvent(md="5_22", name="国际生物多样性日", ename="International Biodiversity Day", importance=1),
    RegEvent(md="5_23", name="国际牛奶日", ename="International Milk Day"),
    RegEvent(md="5_31", name="世界无烟日", ename="World No Smoking Day", importance=1),
    RegEvent(md="6_1", name="儿童节", ename="Children's Day", importance=1),
    RegEvent(md="6_5", name="世界环境日", ename="International Environment Day", importance=1),
    RegEvent(md="6_17", name="世界防治荒漠化和干旱日", ename="World Day to combat desertification"),
    RegEvent(md="6_23", name="国际奥林匹克日", ename="International Olympic Day", importance=1),
    RegEvent(md="6_26", name="国际禁毒日", ename="International Day Against Drug Abuse and Illicit Trafficking"),
    RegEvent(md="7_1", name="建党节"),
    RegEvent(md="7_1", name="国际建筑日", ename="International Architecture Day"),
    RegEvent(md="7_7", name="中国人民抗日战争纪念日"),
    RegEvent(md="7_11", name="世界人口日", ename="World Population Day"),
    RegEvent(md="8_1", name="建军节"),
    RegEvent(md="8_12", name="国际青年节", ename="International Youth Day", importance=1),
    RegEvent(md="9_10", name="教师节", importance=1),
    RegEvent(md="9_16", name="国际臭氧层保护日", ename="International Day for the Preservation of the Ozone Layer"),
    RegEvent(md="9_21", name="世界停火日", ename="World Ceasefire Day", importance=1),
    RegEvent(md="9_27", name="世界旅游日", ename="World Tourism Day"),
    RegEvent(md="10_1", name="国庆节", importance=2, lasts=7),
    RegEvent(md="10_1", name="国际音乐日", ename="International Music Day", importance=1),
    RegEvent(md="10_1", name="国际老年人日", ename="International Day of Older Persons"),
    RegEvent(md="10_4", name="世界动物日", ename="World Animal Day"),
    RegEvent(md="10_5", name="世界教师日", ename="World Teachers' Day"),
    RegEvent(md="10_9", name="世界邮政日", ename="World Post Day"),
    RegEvent(md="10_10", name="世界精神卫生日", ename="World Mental Health Day", importance=1),
    RegEvent(md="10_14", name="世界标准日", ename="World Standards Day"),
    RegEvent(md="10_15", name="国际盲人节", ename="International Day of the Blind"),
    RegEvent(md="10_16", name="世界粮食日", ename="World Food Day", importance=1),
    RegEvent(md="10_17", name="国际消除贫困日", ename="International Day for the Eradication of Poverty"),
    RegEvent(md="10_24", name="联合国日", ename="United Nations Day"),
    RegEvent(md="10_24", name="世界发展新闻日", ename="World Development Information Day"),
    RegEvent(md="10_31", name="万圣节", ename="Halloween", importance=2),
    RegEvent(md="11_8", name="中国记者节"),
    RegEvent(md="11_9", name="消防宣传日"),
    RegEvent(md="11_14", name="世界糖尿病日", ename="World Diabetes Day"),
    RegEvent(md="11_17", name="国际大学生节", ename="International Students' Day"),
    # RegEvent(md="11_24", name="感恩节", ename="Thanksgiving Day", importance=2),
    RegEvent(md="11_25", name="国际消除对妇女的暴力日", ename="International Day For the elimination of Violence against Women"),
    RegEvent(md="12_1", name="世界爱滋病日", ename="World AIDS Day"),
    RegEvent(md="12_3", name="世界残疾人日", ename="World Disabled Day"),
    RegEvent(md="12_9", name="世界足球日", ename="World Football Day"),
    RegEvent(md="12_24", name="平安夜", ename="Christmas Eve", importance=2),
    RegEvent(md="12_25", name="圣诞节", ename="Christmas Day", importance=1), # We give it importance 1 since it's just one day after importance 2
    RegEvent(mwd="5_2_0", name="母亲节", ename="Mother's Day", importance=1),
    RegEvent(mwd="6_3_0", name="父亲节", ename="Father's Day", importance=1),
    RegEvent(mwd="9_3_2", name="国际和平日", ename="International Peace Day", importance=1),
    RegEvent(mwd="10_1_1", name="世界住房日", ename="World Habitat Day"),
    RegEvent(mwd="10_2_1", ename="Canadian Thanksgiving Day", importance=1), # This is too religion specific so we don't represent it to Chinese users
    RegEvent(mwd="10_2_3", name="国际减轻自然灾害日", ename="International Day for Natural Disaster Reduction"),
    RegEvent(mwd="10_2_4", name="世界爱眼日", ename="World Sight Day", importance=1),
    RegEvent(lmd="1_1", name="春节", ename="Chinese Spring Festival", importance=1, lasts=7), # We give it importance 1 since it's just one day after importance 2
    RegEvent(lmd="1_15", name="元宵节", importance=1),
    RegEvent(lmd="5_5", name="端午节", importance=1),
    RegEvent(lmd="7_7", name="七夕节", ename="Chinese Valentine's Day", importance=1),
    RegEvent(lmd="8_15", name="中秋节", importance=1),
    RegEvent(lmd="9_9", name="重阳节", importance=1),
    RegEvent(lmd="12_8", name="腊八节", importance=1),
    # RegEvent(lmd="12_30", name="除夕", importance=2, lasts=1),
]

EventsCollection._add_basic(events_list)

if __name__ == '__main__':

    e = EventsCollection()
    print(e._find_qing_ming(2026))
    # print(e._get_last_weekday(2025, 11, 4))
    # print(e._get_last_ld(2025, 12))
    # days = e.find(2025, 1, 31)
    # print(days)
    # for day in days:
    #     print(str(day))


import datetime
import pytz
import json
import re
import traceback
from maica.mtools import *
from .mfocus_main import SfBoundCoroutine
from maica.maica_utils import *

class AgentTools():
    """I didn't intend to make classes everywhere, but they're oh so convenient."""
    def __init__(self, fsc: FullSocketsContainer, sf_inst: SfBoundCoroutine):
        self.fsc, self.sf_inst = fsc, sf_inst

    def _time_tz(self, tz="zh"):
        if tz == 'zh':
            tz = "Asia/Shanghai"
        elif tz == 'en':
            tz = "America/Indiana/Vincennes"
        try:
            time_now = datetime.datetime.now(tz=pytz.timezone(tz))
        except Exception:
            time_now = datetime.datetime.now()
        return time_now

    async def time_acquire(self, *args, **kwargs) -> tuple[str, str]:
        """Gets current time. Requires fsc."""
        target_lang = self.fsc.maica_settings.basic.target_lang
        tz = self.fsc.maica_settings.extra.tz
        time = self._time_tz(tz or target_lang)
        match time:
            case time if time.hour < 4:
                time_range = '半夜' if target_lang == 'zh' else 'at midnight'
            case time if 4 <= time.hour < 6:
                time_range = '凌晨' if target_lang == 'zh' else 'before dawn'
            case time if 6 <= time.hour < 8:
                time_range = '早上' if target_lang == 'zh' else 'at dawn'
            case time if 8 <= time.hour < 11:
                time_range = '上午' if target_lang == 'zh' else 'in morning'
            case time if 11 <= time.hour < 13:
                time_range = '中午' if target_lang == 'zh' else 'at noon'
            case time if 13 <= time.hour < 18:
                time_range = '下午' if target_lang == 'zh' else 'in afternoon'
            case time if 18 <= time.hour < 23:
                time_range = '晚上' if target_lang == 'zh' else 'at night'
            case time if 23 <= time.hour:
                time_range = '深夜' if target_lang == 'zh' else 'at midnight'
        time_friendly = f"现在是{time_range}{time.hour}点{str(time.minute).zfill(2)}分" if target_lang == 'zh' else f"It's now {str(time.hour).zfill(2)}:{str(time.minute).zfill(2)} {time_range}"
        content = f'{str(time.hour).zfill(2)}:{str(time.minute).zfill(2)}'
        return content, time_friendly

    async def date_acquire(self, *args, **kwargs) -> tuple[str, str]:
        """Gets current date. Requires fsc and sf_inst."""
        target_lang = self.fsc.maica_settings.basic.target_lang
        tz = self.fsc.maica_settings.extra.tz
        date = self._time_tz(tz or target_lang)
        weeklist = ["星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日"] if target_lang == 'zh' else ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
        weekday = weeklist[date.weekday()]
        south = self.sf_inst.read_from_sf('_mas_pm_live_south_hemisphere')
        if south:
            match date:
                case date if 3 <= date.month < 6:
                    season = '秋季' if target_lang == 'zh' else 'autumn'
                case date if 6 <= date.month < 9:
                    season = '冬季' if target_lang == 'zh' else 'winter'
                case date if 9 <= date.month < 12:
                    season = '春季' if target_lang == 'zh' else 'spring'
                case date if 12 <= date.month or date.month < 3:
                    season = '夏季' if target_lang == 'zh' else 'summer'
        else:
            match date:
                case date if 3 <= date.month < 6:
                    season = '春季' if target_lang == 'zh' else 'spring'
                case date if 6 <= date.month < 9:
                    season = '夏季' if target_lang == 'zh' else 'summer'
                case date if 9 <= date.month < 12:
                    season = '秋季' if target_lang == 'zh' else 'autumn'
                case date if 12 <= date.month or date.month < 3:
                    season = '冬季' if target_lang == 'zh' else 'winter'
        date_friendly = f"今天是{date.year}年{season}{date.month}月{date.day}日{weekday}" if target_lang == 'zh' else f"Today is {date.year}.{date.month}.{date.day} {season}, {weekday}"
        content = f'{date.year}.{date.month}.{date.day}, {weekday}'
        return content, date_friendly

    async def weather_acquire(self, *args: list[str], **kwargs: dict[str: str]) -> tuple[str, str]:
        """Gets current weather. Requires fsc and (sf_inst or location)."""
        location = args[0] if args else kwargs.get('location')
        target_lang = self.fsc.maica_settings.basic.target_lang
        weather = None

        if not location:
            location = self.sf_inst.read_from_sf('mas_geolocation')

        if location:
            try:
                weather = await weather_api_get(location)
                content = json.dumps(weather, ensure_ascii=False)
                weather_friendly = f"当前气温是{weather['temperature']}度, 当前天气是{weather['weather']}, 当前湿度是{weather['humidity']}%" if target_lang == 'zh' else f"Current temperature is {weather['temperature']} degrees celsius, current weather is {weather['weather']}, current humidity is {weather['humidity']} percent"
            except CommonMaicaException as ce:
                await messenger(None, 'maica_mfocus_weather_failed', traceray_id=self.fsc.rsc.traceray_id, error=ce)

        if not weather:
            content = '天气未知' if target_lang == 'zh' else "Weather unknown"
            weather_friendly = None
        return content, weather_friendly

    async def event_acquire(self, *args, **kwargs: dict[str: int]) -> tuple[str, str]:
        """Gets meaningful events. Requires fsc and sf_inst, optional ymd and predict and importance."""
        target_lang = self.fsc.maica_settings.basic.target_lang
        tz = self.fsc.maica_settings.extra.tz
        time_today = self._time_tz(tz or target_lang)
        date_cursor = EventsCollection()

        date_in = []
        for k in 'year', 'month', 'day':
            date_in.append(kwargs[k] if kwargs.get(k) else getattr(time_today, k))

        try:
            player_bday = self.sf_inst.read_from_sf('mas_player_bday')
            player_bday = datetime.datetime(*vali_date(*player_bday))
        except Exception:
            player_bday = None

        major_day_is_today = False
        target_date = datetime.datetime(*vali_date(*date_in))

        predict = kwargs.get('predict')
        if is_today(target_date):
            major_day_is_today = True
            if predict is None:
                predict = 2
        elif predict is None:
            predict = 0

        # The importance mechanism:

        # When major day is today, we want to search a wider range of days, 
        # but ignore the unimportant events since they aren't cared in daily
        # lives

        # But when major day is not today, we assume that user is asking a
        # question about a percise date, so we'll dig deeper on that. The
        # following days are usually not cared about

        # Specifically, when tnd_aggressive is set to >= 3, we enable all
        # importance levels to be represented on major day. This is not
        # recommended

        global_importance = kwargs.get('importance', 1)
        if not major_day_is_today:
            global_importance -= 1
        if self.fsc.maica_settings.extra.tnd_aggressive >= 3:
            global_importance -= 1
        assert isinstance(global_importance, int), "Importance input not valid"

        days_regs_list: list[list[RegEvent]] = []
        days_events_list = []
        for next_days in range(predict + 1):

            target_next_day = target_date + datetime.timedelta(days=next_days)
            days_regs_list.append(date_cursor.find(target_next_day.year, target_next_day.month, target_next_day.day))

        day_seq = 0
        for day_regs_list in days_regs_list:
            day_seq += 1
            is_major_day = day_seq == 1
            local_importance = global_importance if is_major_day else global_importance + 1

            if major_day_is_today:
                match day_seq:
                    case 1:
                        today = "今天" if target_lang == 'zh' else "Today"
                    case 2:
                        today = "明天" if target_lang == 'zh' else "Tomorrow"
                    case 3:
                        today = "后天" if target_lang == 'zh' else "The day after tomorrow"
                    case _:
                        today = f"{day_seq - 1}天后" if target_lang == 'zh' else f"{day_seq - 1} days later"
            else:
                match day_seq:
                    case 1:
                        today = "这一天" if target_lang == 'zh' else "This day"
                    case 2:
                        today = f"这一天后{day_seq - 1}天" if target_lang == 'zh' else f"{day_seq - 1} day after this day"
                    case _:
                        today = f"这一天后{day_seq - 1}天" if target_lang == 'zh' else f"{day_seq - 1} days after this day"

            day = target_date + datetime.timedelta(days = day_seq - 1)
            day_events_list = []

            # Here we determine player's bday
            if player_bday:
                player_age = day.year - player_bday.year
                if (day.month, day.day) == (player_bday.month, player_bday.day):
                    # Happy birthday [player]!
                    day_events_list.append(f"[player]的{player_age}岁生日" if target_lang == 'zh' else f"[player]'s {add_seq_suffix(player_age)} birthday")

            # Here we determine Monika's bday
            if (day.month, day.day) == (9, 22):
                # Happy birthday Monika!
                day_events_list.append(f"莫妮卡的生日" if target_lang == 'zh' else f"Monika's birthday")

            # Here we handle every reg for today
            for reg in day_regs_list:

                # We ignore the unimportant regs
                if reg.importance >= local_importance:
                    if target_lang == 'zh':
                        if reg.name:
                            day_events_list.append(reg.name)
                    else:

                        # Most Chinese traditional festivals are not registered with ename
                        # We don't add them since English users likely don't care
                        if reg.ename:
                            day_events_list.append(reg.ename)

            if day_events_list:
                today_is = f"{today}是" if target_lang == 'zh' else f"{today} is "
                joint = ", 也是" if target_lang == 'zh' else ", and also "
                day_event = today_is + joint.join(day_events_list)

                days_events_list.append(day_event)
            elif is_major_day:
                days_events_list.append(f"{today}不是特殊节日" if target_lang == 'zh' else f"{today} is not a special event or holiday")

        if days_events_list:
            content = days_event = '; '.join(days_events_list)
        else:
            match [major_day_is_today, target_lang]:
                case [p, t] if p and t == 'zh':
                    d0_this_day = "今天"
                case [p, t] if p and t == 'en':
                    d0_this_day = "Today"
                case [p, t] if not p and t == 'zh':
                    d0_this_day = "这一天"
                case [p, t] if not p and t == 'en':
                    d0_this_day = "This day"
            content = days_event = f"{major_day_is_today}不是特殊节日" if target_lang == 'zh' else f"{major_day_is_today} is not a special event or holiday"

        return content, days_event

    async def persistent_acquire(self, *args: list[str], **kwargs: dict[str: str]) -> tuple[str, str]:
        """Gets value from persistent. Requires fsc and sf_inst and query."""
        query = args[0] if args else kwargs.get('query')
        target_lang = self.fsc.maica_settings.basic.target_lang
        response_json = await self.sf_inst.mfocus_find_info(query)
        if response_json:
            content = json.dumps(response_json, ensure_ascii=False)
        else:
            content = '没有相关信息' if target_lang == 'zh' else "No related information found"
        return content, response_json

    async def search_internet(self, *args: list[str], **kwargs: dict[str: str]) -> tuple[str, str]:
        """Searches result from internet. Requires fsc and location_req and query and original_query, optional sf_inst."""
        query = args[0] if args else kwargs.get('query')
        location_req = args[1] if args else kwargs.get('location_req')
        original_query = args[2] if args else kwargs.get('original_query')
        target_lang = self.fsc.maica_settings.basic.target_lang

        if location_req:
            try:
                geolocation = self.sf_inst.read_from_sf('mas_geolocation')
            except Exception:
                geolocation = None
        else:
            geolocation = None
        if geolocation:
            query = geolocation + query
            original_query = geolocation + original_query

        return await internet_search(self.fsc, query, original_query)

if __name__ == "__main__":
    import asyncio
    import time
    async def main():
        fsc = FullSocketsContainer(maica_settings=MaicaSettings())
        fsc.maica_settings.verification.update(user_id=18064, username="edge1")
        fsc.maica_settings.update(target_lang='en')
        fsc.maica_pool = await ConnUtils.maica_pool()
        fsc.mfocus_conn = await ConnUtils.mfocus_conn()
        # fsc.maica_settings.update(esc_aggressive=False)
        sf_inst = await SfBoundCoroutine.async_create(fsc)
        at = AgentTools(fsc, sf_inst)
        # res_d = asyncio.run(at.date_acquire())
        # res_e = asyncio.run(at.event_acquire(year=2025, month=10, day=1))
        # res_i = asyncio.run(at.search_internet('24年奥运会', False, '24年奥运会在哪里'))
        time0 = time.time()
        res_p = await at.persistent_acquire('你喜欢吃什么')
        print(f'Time consumed: {time.time() - time0}')
        print(res_p)
    asyncio.run(main())


import re
import json
import traceback
from . import wiki_scraping
from openai import AsyncOpenAI
from maica.maica_utils import *

async def make_inspire(title_in=None, target_lang='zh'):
    success = True
    exception = None
    try:
        title, summary = await wiki_scraping.get_page(title_in, target_lang)
        if not summary or summary.isspace():
            raise Exception('MSpire got no return')
        if target_lang == 'zh':
            prompt = f"利用提供的以下信息, 主动阐明话题并简单地聊聊{re.sub('_', '', title)}:\n\n{summary}\n\n你不必在输出中包含内容的全部信息, 但应当融入自己的理解与思考. 你的回答应当有开头招呼, 信息简述, 思考理解, 总结收尾."
        else:
            prompt = f"Summarize the topic and talk about {re.sub('_', '', title)} briefly with me using provided informations below: {summary}\nYou don't have to include every information provided, but you should combine your own thinking and understanding into your outputs. Your reply should contain a greeting, brief introduction, thoughts and opinions, and an ending."
        #message = json.dumps({"role": "user", "content": prompt}, ensure_ascii=False)
        hash_identity = await hash_sha256(prompt.encode())
        #print(hash_identity)
    except Exception as excepted:
        #traceback.print_exc()
        success = False
        exception = excepted
        prompt = None
        hash_identity = None
    #print(summary)
    return success, exception, prompt, hash_identity


if __name__ == '__main__':
   import asyncio
   print(asyncio.run(make_inspire(target_lang='zh')))


import os
from maica.maica_utils import load_env, get_inner_path

mark_path = get_inner_path('.initialized')

def check_marking():

    try:
        with open(mark_path, 'r'):
            return True
    except:
        return False
    
def create_marking():
    curr_version, legc_version = load_env('VERSION_CONTROL').split(';', 1)
    with open(mark_path, 'w') as mark:
        mark.write(f"This file's existence indicates that the program has been initiated once.\nTo try initiating it again, delete this file.\n\nWarning: Deleting this file will not make the program run any cleanups.\n\nInitiation version: {curr_version}")
    return

if __name__ == "__main__":
    print(check_marking())
    create_marking()
    print(check_marking())


import asyncio
import os
from maica.maica_utils import DbPoolCoroutine, ConnUtils
from typing import *
from maica.maica_utils import *

async def create_tables():

    AUTH_DB = load_env('AUTH_DB')
    MAICA_DB = load_env('MAICA_DB')

    basic_pool: DbPoolCoroutine = await ConnUtils.basic_pool()
    auth_created = False

    if basic_pool:
        # We have to create databases before tables
        result = await basic_pool.query_get("SHOW DATABASES", fetchall=True)
        curr_dbs = []
        for curr_db in result:
            curr_dbs.append(curr_db[0])

        if not AUTH_DB in curr_dbs:
            print(f"[maica-dbs-init] AUTH_DB {AUTH_DB} does not exist, creating...")
            await basic_pool.query_modify(f"CREATE DATABASE IF NOT EXISTS {AUTH_DB}")
            auth_created = True
        else:
            print(f"[maica-dbs-init] AUTH_DB {AUTH_DB} exists, skipping...")

        if not MAICA_DB in curr_dbs:
            print(f"[maica-dbs-init] MAICA_DB {MAICA_DB} does not exist, creating...")
            await basic_pool.query_modify(f"CREATE DATABASE IF NOT EXISTS {MAICA_DB}")
        else:
            print(f"[maica-dbs-init] MAICA_DB {MAICA_DB} exists, skipping...")
    elif not os.path.exists(get_inner_path(AUTH_DB)):
        auth_created = True

    auth_pool = await ConnUtils.auth_pool(ro=False)
    maica_pool = await ConnUtils.maica_pool()

    auth_tables = [
"""
CREATE TABLE IF NOT EXISTS `users` (
`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
`username` varchar(100) COLLATE utf8mb4_unicode_ci NOT NULL,
`nickname` varchar(255) COLLATE utf8mb4_unicode_ci DEFAULT NULL,
`email` varchar(150) COLLATE utf8mb4_unicode_ci NOT NULL,
`is_email_confirmed` tinyint(1) NOT NULL DEFAULT '0',
`password` varchar(100) COLLATE utf8mb4_unicode_ci NOT NULL,
PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci
"""
    ] if basic_pool else [
"""
CREATE TABLE IF NOT EXISTS users (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    username TEXT NOT NULL,
    nickname TEXT DEFAULT NULL,
    email TEXT NOT NULL,
    is_email_confirmed INTEGER NOT NULL DEFAULT 0,
    password TEXT NOT NULL
);
"""
    ]
    maica_tables = [
"""
CREATE TABLE IF NOT EXISTS `account_status` (
`user_id` int(11) NOT NULL,
`status` longtext DEFAULT NULL,
`preferences` longtext DEFAULT NULL,
PRIMARY KEY (`user_id`)
)
""",
"""
CREATE TABLE IF NOT EXISTS `cchop_archived` (
`archive_id` int(11) NOT NULL AUTO_INCREMENT,
`chat_session_id` int(11) NOT NULL,
`content` longtext DEFAULT NULL,
`archived` int(11) DEFAULT NULL,
PRIMARY KEY (`archive_id`)
)
""",
"""
CREATE TABLE IF NOT EXISTS `chat_session` (
`chat_session_id` int(11) NOT NULL AUTO_INCREMENT,
`user_id` int(11) NOT NULL,
`chat_session_num` int(11) NOT NULL,
`content` longtext DEFAULT NULL,
PRIMARY KEY (`chat_session_id`)
)
""",
"""
CREATE TABLE IF NOT EXISTS `csession_archived` (
`archive_id` int(11) NOT NULL AUTO_INCREMENT,
`chat_session_id` int(11) NOT NULL,
`content` longtext DEFAULT NULL,
PRIMARY KEY (`archive_id`)
)
""",
"""
CREATE TABLE IF NOT EXISTS `persistents` (
`persistent_id` int(11) NOT NULL AUTO_INCREMENT,
`user_id` int(11) NOT NULL,
`chat_session_num` int(11) NOT NULL,
`content` longtext DEFAULT NULL,
`timestamp` datetime on update CURRENT_TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
PRIMARY KEY (`persistent_id`)
)
""",
"""
CREATE TABLE IF NOT EXISTS `triggers` (
`trigger_id` int(11) NOT NULL AUTO_INCREMENT,
`user_id` int(11) NOT NULL,
`chat_session_num` int(11) NOT NULL,
`content` longtext DEFAULT NULL,
`timestamp` datetime on update CURRENT_TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
PRIMARY KEY (`trigger_id`)
)
""",
"""
CREATE TABLE IF NOT EXISTS `ms_cache` (
`spire_id` int(11) NOT NULL AUTO_INCREMENT,
`hash` longtext NOT NULL,
`content` longtext DEFAULT NULL,
`timestamp` datetime on update CURRENT_TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
PRIMARY KEY (`spire_id`)
)
"""
    ] if basic_pool else [
"""
CREATE TABLE IF NOT EXISTS account_status (
    user_id INTEGER PRIMARY KEY NOT NULL,
    status TEXT DEFAULT NULL,
    preferences TEXT DEFAULT NULL
)
""",
"""
CREATE TABLE IF NOT EXISTS cchop_archived (
    archive_id INTEGER PRIMARY KEY AUTOINCREMENT,
    chat_session_id INTEGER NOT NULL,
    content TEXT DEFAULT NULL,
    archived INTEGER DEFAULT NULL
)
""",
"""
CREATE TABLE IF NOT EXISTS chat_session (
    chat_session_id INTEGER PRIMARY KEY AUTOINCREMENT,
    user_id INTEGER NOT NULL,
    chat_session_num INTEGER NOT NULL,
    content TEXT DEFAULT NULL
)
""",
"""
CREATE TABLE IF NOT EXISTS csession_archived (
    archive_id INTEGER PRIMARY KEY AUTOINCREMENT,
    chat_session_id INTEGER NOT NULL,
    content TEXT DEFAULT NULL
)
""",
"""
CREATE TABLE IF NOT EXISTS persistents (
    persistent_id INTEGER PRIMARY KEY AUTOINCREMENT,
    user_id INTEGER NOT NULL,
    chat_session_num INTEGER NOT NULL,
    content TEXT DEFAULT NULL,
    timestamp DATETIME DEFAULT CURRENT_TIMESTAMP
)
""",
"""
CREATE TABLE IF NOT EXISTS triggers (
    trigger_id INTEGER PRIMARY KEY AUTOINCREMENT,
    user_id INTEGER NOT NULL,
    chat_session_num INTEGER NOT NULL,
    content TEXT DEFAULT NULL,
    timestamp DATETIME DEFAULT CURRENT_TIMESTAMP
)
""",
"""
CREATE TABLE IF NOT EXISTS ms_cache (
    spire_id INTEGER PRIMARY KEY AUTOINCREMENT,
    hash TEXT NOT NULL,
    content TEXT DEFAULT NULL,
    timestamp DATETIME DEFAULT CURRENT_TIMESTAMP
)
""",
"""
CREATE TRIGGER IF NOT EXISTS update_timestamp_persistents
AFTER UPDATE ON persistents
FOR EACH ROW
BEGIN
    UPDATE persistents
    SET timestamp = CURRENT_TIMESTAMP
    WHERE persistent_id = OLD.persistent_id;
END;
""",
"""
CREATE TRIGGER IF NOT EXISTS update_timestamp_triggers
AFTER UPDATE ON triggers
FOR EACH ROW
BEGIN
    UPDATE triggers
    SET timestamp = CURRENT_TIMESTAMP
    WHERE trigger_id = OLD.trigger_id;
END;
""",
"""
CREATE TRIGGER IF NOT EXISTS update_timestamp_ms_cache
AFTER UPDATE ON ms_cache
FOR EACH ROW
BEGIN
    UPDATE ms_cache
    SET timestamp = CURRENT_TIMESTAMP
    WHERE spire_id = OLD.spire_id;
END;
""",
    ]

    # Notice: These triggers act as 'on update CURRENT_TIMESTAMP', since
    # there is no convenient way for this in SQLite.

    if basic_pool:
        for table in maica_tables:
            table += " ENGINE=InnoDB DEFAULT CHARSET=utf8mb4"

    if auth_created:
        for table in auth_tables:
            print("[maica-dbs-init] Adding table to AUTH_DB...")
            await auth_pool.query_modify(table)
    else:
        print("[maica-dbs-init]\nWarning: AUTH_DB was not created by MAICA, so we're not writing anything for security reason.\nPlease make sure AUTH_DB is already ready for authentication.")

    for table in maica_tables:
        print("[maica-dbs-init] Adding table to MAICA_DB...")
        await maica_pool.query_modify(table)

if __name__ == "__main__":
    asyncio.run(create_tables())


import asyncio
import platform
import colorama

from maica.maica_utils import messenger, load_env, MsgType


def basic_chk():
    sysstruct = platform.system()
    assert sysstruct in ['Windows', 'Linux'], 'Your system is not supported!'

    curr_version, legc_version = load_env('VERSION_CONTROL').split(';', 1)
    asyncio.run(messenger(info=f"Running MAICA Illuminator V{curr_version} on {sysstruct}", type=MsgType.PRIM_SYS))

    try:
        proxyaddr = load_env('PROXY_ADDR')
    except Exception:
        proxyaddr = ''
    if proxyaddr:
        print(f"Global proxy detected, using {proxyaddr}")

    else:
        print("Global proxy absent")


import os
import sys
from pathlib import Path
from dotenv import load_dotenv
try:
    from maica import maica_ws
except Exception:
    current_dir = Path(__file__).parent
    parent_dir = current_dir.parent
    if str(parent_dir) not in sys.path:
        sys.path.insert(0, str(parent_dir))

import asyncio
import argparse

from maica import maica_ws, maica_http, common_schedule
from maica.maica_utils import *
from maica.initializer import *

from maica.maica_http import pkg_init_maica_http
from maica.maica_nows import pkg_init_maica_nows
from maica.maica_utils import pkg_init_maica_utils
def pkg_init_maica():
    pkg_init_maica_http()
    pkg_init_maica_nows()
    pkg_init_maica_utils()

initialized = False

def check_params(envdir=None):
    """This will only run once. Recalling will not take effect."""
    global initialized
    def init_parser():
        parser = argparse.ArgumentParser(description="Start MAICA Illuminator deployment")
        parser.add_argument('-e', '--envdir', help='Include external env file for running deployment')
        return parser
    def dest_env(envdir=None):
        parser = init_parser()
        args = parser.parse_args()
        envdir = envdir or args.envdir
        try:
            if envdir:
                realpath = os.path.abspath(envdir)
                if not os.path.isfile(realpath):
                    print(f'[maica-argparse] Warning: {realpath} is not a file, trying {os.path.join(realpath, '.env')}...')
                    realpath = os.path.join(realpath, '.env')
                    if not os.path.isfile(realpath):
                        raise Exception('designated env file not exist')
                print(f'[maica-argparse] Loading env file {realpath}...')
                load_dotenv(dotenv_path=realpath)

            else:
                realpath = get_inner_path('.env')
                print(f'[maica-argparse] No env file designated, trying to load {realpath}...')
                if os.path.isfile(realpath):
                    load_dotenv(dotenv_path=realpath)
                else:
                    print(f'[maica_argparse] Warning: {realpath} is not a file, skipping real env file...')
            
            if not load_env('IS_REAL_ENV') == '1':
                realpath = get_inner_path('env_example')
                print(f'[maica-argparse] Trying to load {realpath} to maintain basic functions...')
                if os.path.isfile(realpath):
                    load_dotenv(dotenv_path=realpath)
                else:
                    raise Exception('no env file available')
        except Exception as e:
            print(f'[maica-argparse] Error: {str(e)}, quitting...')
            exit(1)

    if not initialized:
        dest_env()
        pkg_init_maica()
        initialized = True

def check_basic_init():
    if load_env('IS_REAL_ENV') == '1':
        return
    else:
        print('''No real env detected, is this workflow?
If it is, at least the imports and grammar are good if you see this.
If not:
    If you're running MAICA for deployment, pass in "--envdir path/to/.env".
    If you're developing with MAICA as dependency, call maica.init() after import.
    Or, you can manually set the necessary env vars.
Quitting...'''
              )
        quit(0)

def check_init():
    if not check_marking():
        generate_rsa_keys()
        asyncio.run(create_tables())
        create_marking()
        asyncio.run(messenger(info="MAICA Illuminator initiation finished", type=MsgType.PRIM_SYS))
    else:
        asyncio.run(messenger(info="Initiated marking detected, skipping initiation", type=MsgType.DEBUG))

async def start_all():
    if load_env('DB_ADDR') == 'sqlite':
        await messenger(info='\nMAICA using SQLite database detected\nWhile MAICA Illuminator is fully compatible with SQLite, it can be a significant drawback of performance under heavy workload\nIf this is a public service instance, it\'s strongly recommended to use MySQL/MariaDB instead', type=MsgType.DEBUG)
    if not load_env('PROXY_ADDR'):
        await messenger(info='\nInternet proxy absence detected\nMAICA Illuminator needs to access Google and Wikipedia for full functionalities, so you\'ll possibly need a proxy for those\nYou can ignore this message safely if you have direct access to those or have a global proxy set already', type=MsgType.DEBUG)
    if load_env('DEV_STATUS') != 'serving':
        await messenger(info='\nServer status not serving detected\nWith this announcement taking effect, standard clients will stop further communications with MAICA Illuminator respectively\nYou should set DEV_STATUS to \'serving\' whenever the instance is ready to serve', type=MsgType.DEBUG)
    if (load_env('MCORE_NODE') and load_env('MCORE_USER') and load_env('MCORE_PWD')) or (load_env('MFOCUS_NODE') and load_env('MFOCUS_USER') and load_env('MFOCUS_PWD')):
        await messenger(info='\nOne or more NVwatch nodes detected\nNVwatch of MAICA Illuminator will try to collect nvidia-smi outputs through SSH, which can fail the process if SSH not avaliable\nIf SSH of nodes are not accessable or not wanted to be used, delete X_NODE, X_USER, X_PWD accordingly', type=MsgType.DEBUG)

    auth_pool, maica_pool = await asyncio.gather(ConnUtils.auth_pool(), ConnUtils.maica_pool())
    kwargs = {"auth_pool": auth_pool, "maica_pool": maica_pool}
    task_ws = asyncio.create_task(maica_ws.prepare_thread(**kwargs))
    task_http = asyncio.create_task(maica_http.prepare_thread(**kwargs))
    task_schedule = asyncio.create_task(common_schedule.schedule_rotate_cache(**kwargs))

    await asyncio.wait([
        task_ws,
        task_http,
        task_schedule,
    ], return_when=asyncio.FIRST_COMPLETED)

    await messenger(info="All quits collected, doing final cleanup...", type=MsgType.DEBUG)
    await asyncio.gather(auth_pool.close(), maica_pool.close(), return_exceptions=True)
    quit()

def full_start():
    check_params()
    check_basic_init()
    check_init()
    asyncio.run(start_all())

if __name__ == "__main__":
    full_start()